#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for running the clustering analysis on a matrix.

@author: mark
"""

import logging
import scipy.cluster.hierarchy as cluster

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def clustering_by_all_methods(condensed_distance_matrix):
    """This carries out the clustering using all the clustering methods.
    A dictionary is returned where the key is the clustering approach used.
    """
    ward_linkage = ward_clustering(condensed_distance_matrix)
    wpgmc_linkage = wpgmc_clustering(condensed_distance_matrix)
    upgmc_linkage = upgmc_clustering(condensed_distance_matrix)
    wpgma_linkage = wpgma_clustering(condensed_distance_matrix)
    upgma_linkage = upgma_clustering(condensed_distance_matrix)
    max_linkage = max_clustering(condensed_distance_matrix)
    min_linkage = min_clustering(condensed_distance_matrix)
    return {
        "ward_clustering": ward_linkage,
        "wpgmc_clustering": wpgmc_linkage,
        "upgmc_clustering": upgmc_linkage,
        "wpgma_clustering": wpgma_linkage,
        "upgma_clustering": upgma_linkage,
        "max_clustering": max_linkage,
        "min_clustering": min_linkage,
    }


def ward_clustering(condensed_distance_matrix):
    """This calculates the ward linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "ward")


def wpgmc_clustering(condensed_distance_matrix):
    """This calculates the median/WPGMC linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "median")


def upgmc_clustering(condensed_distance_matrix):
    """This calculates the centroid/UPGMC linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "centroid")


def wpgma_clustering(condensed_distance_matrix):
    """This calculates the weighted/WPGMA linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "weighted")


def upgma_clustering(condensed_distance_matrix):
    """This calculates the average/UPGMA linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "average")


def max_clustering(condensed_distance_matrix):
    """This calculates the complete/max/farthest linkage for clustering.
    """
    return calculate_clustering(condensed_distance_matrix, "complete")


def min_clustering(condensed_distance_matrix):
    """This calculates the single/min/nearest linkage for clustering
    """
    return calculate_clustering(condensed_distance_matrix, "single")


def calculate_clustering(condensed_distance_matrix, method):
    """This calculates the clustering for the given input method.
    """
    return cluster.linkage(condensed_distance_matrix, method=method)
