#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script with functionality for collating polynomial information, and output to
csv, so it can be converted to a latex table.
@author: mark
"""

import logging
import pandas
import csv
import re
import solventmapcreator.io.polynomialdatareader as polyread

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)


def compact_polynomial_files(
    filename_list, polynomial_order, output_filename_stem, **kwargs
):
    """This compacts the information in the differnet polynomial files into a single file.
    """
    poly_data = read_polyfiles(filename_list, **kwargs)
    sorted_data = create_sorted_list(poly_data, polynomial_order, **kwargs)
    sorted_data = create_header_line(polynomial_order) + sorted_data
    write_info_to_csv_file(output_filename_stem + ".csv", sorted_data)
    convert_to_tex_table(output_filename_stem + ".csv", output_filename_stem + ".tex")
    return 0


def convert_to_tex_table(csv_filename, tex_filename):
    """This converts the csv file to a latex table and writes to file.
    """
    dframe = pandas.read_csv(csv_filename, delimiter="\t")
    dframe.to_latex(buf=tex_filename, index=False, longtable=True)


def write_info_to_csv_file(filename, outputdata):
    """This writes the information to file.
    """
    with open(filename, "w") as fileout:
        csv_writer = csv.writer(fileout, delimiter="\t",lineterminator="\n")
        csv_writer.writerows(outputdata)
    return 0


def create_header_line(polynomial_order):
    """This creates the header line for the file.
    """
    return [
        ["Molecule Name", "Region"]
        + ["{}th".format(order) for order in range(polynomial_order + 1)]
    ]


def create_sorted_list(poly_data_by_id, polynomial_order, **kwargs):
    """This creates a sorted list of the values.
    """
    sorted_data = []
    for mol_id in sorted(poly_data_by_id.keys()):
        if kwargs.get("solv_type", None) == "binary":
            binary_pattern = re.compile(r"\d\.\d{3}")
            mol_name = "{:.0f}".format(
                float(re.findall(binary_pattern, mol_id)[0]) * 100
            )
        else:
            mol_name = mol_id
        pos_line = [mol_name, "positive"] + list(
            poly_data_by_id[mol_id][polynomial_order]["positive"]["coefficients"]
        )
        sorted_data.append(pos_line)
        neg_line = [mol_name, "negative"] + list(
            poly_data_by_id[mol_id][polynomial_order]["negative"]["coefficients"]
        )
        sorted_data.append(neg_line)
    return sorted_data


def read_polyfiles(filename_list, **kwargs):
    """This reads in polynomial files.
    """
    return polyread.parse_polynomial_data_file_list(filename_list, **kwargs)
