#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for extracting and splitting up solvation energies based on solvent.
This is to produce more manageable files from the full information dump from
the phasetransfer code.

@author: mark
"""

import logging
import copy
from lxml import etree

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)

PHASE_NAMESPACE_DICT = {"phase": "http://www-hunter.ch.cam.ac.uk/PhaseSchema"}

PHASE = "{{{}}}".format(PHASE_NAMESPACE_DICT["phase"])

def extract_and_write_energy_values_from_files(solvent_filename, energy_xml_filename, energy_type, directory):
    """Extracts energies and splits to different file for each solvent.

    Parameters
    ----------
    solvent_filename : str
        Solvent XML filename.
    energy_xml_filename : str
        Energy XML filename.
    energy_type : str
        type of energy to extract. Either "binding" or "free".
    directory : str
        directory name.

    Returns
    -------
    list of str
        output filenames.

    """
    solvent_id_list = read_in_solvent_id_list(solvent_filename)
    energy_xml = read_xml_file(energy_xml_filename)
    return extract_and_write_energy_values_for_solvent_list(solvent_id_list,
                                                            energy_xml,
                                                            energy_type,
                                                            directory)


def read_in_solvent_id_list(solvent_filename):
    """Read the solvent XML, and extracts the solvent IDs.

    Parameters
    ----------
    solvent_filename : str
        Solvent XML filename.

    Returns
    -------
    solvent_ids : list of str
        List of Solvent IDs.

    """
    solvent_list_element_tree = read_xml_file(solvent_filename)
    return get_solvent_id_list(solvent_list_element_tree)


def get_solvent_id_list(solvent_list_element_tree):
    """Get the solventIDs from the solvent list element.

    Parameters
    ----------
    solvent_list_element_tree : etree.ElementTree
        Element tree representation of solvent XML file.

    Returns
    -------
    solvent_ids : list of str
        List of Solvent IDs.

    """
    xpath_expression = (
        "/phase:SolventList/phase:Solvents/phase:Solvent/@phase:solventID"
    )
    solvent_ids = solvent_list_element_tree.xpath(
        xpath_expression, namespaces=PHASE_NAMESPACE_DICT
    )
    LOGGER.info("Solvent IDs found %i", len(solvent_ids))
    return solvent_ids


def extract_and_write_energy_values_for_solvent_list(
    solvent_id_list, energy_xml, energy_type, directory
):
    """Extract energies for each solvent and write to a new file per solvent.

    Parameters
    ----------
    solvent_id_list : list of str
        list of solvent IDs.
    energy_xml_filename : str
        Energy XML filename.
    energy_type : str
        type of energy to extract. Either "binding" or "free".
    directory : str
        directory name.

    Returns
    -------
    output_files : list of str
        filename list.

    """
    output_files = []
    for solvent_id in solvent_id_list:
        outfilename = extract_and_write_energy_values_for_solvent(
            solvent_id, energy_xml, energy_type, directory
        )
        output_files.append(outfilename)
    return output_files


def extract_and_write_energy_values_for_solvent(
    solvent_id, energy_xml, energy_type, directory
):
    """This extracts the elements and writes the energies found ot a new file.
    """
    energy_values_element = generate_energy_values_element_for_solvent(
        solvent_id, energy_xml, energy_type
    )
    element_tree = create_element_tree(energy_values_element)
    if directory != "":
        directory = directory +"/" if directory[-1] != "/" else directory
    outfilename = directory + create_output_filename(solvent_id, energy_type)
    write_element_tree_to_file(element_tree, outfilename)
    return outfilename

def generate_energy_values_element_for_solvent(solvent_id, energy_xml, energy_type):
    """This extracts the elements for the solvents and creates a new EnergyValues element.
    """
    if energy_type == "binding":
        xpath_expression = generate_binding_energy_xpath_expression(solvent_id)
        energy_elements = get_energy_elements(energy_xml, xpath_expression)
        return generate_energy_values_element(energy_elements, energy_type)
    elif energy_type == "free":
        xpath_expression = generate_free_energy_xpath_expression(solvent_id)
        energy_elements = get_energy_elements(energy_xml, xpath_expression)
        return generate_energy_values_element(energy_elements, energy_type)


def create_output_filename(to_solvent_id, energy_type):
    """This creates an output filename based on the solvent id and energy type.
    """
    filename_stem = to_solvent_id.replace(",", "_")
    filename_stem += energy_type
    return filename_stem + ".xml"


def write_element_tree_to_file(element_tree, filename):
    """Writes element tree to file.
    """
    element_tree.write(
        filename, encoding="UTF-8", xml_declaration=True, pretty_print=True
    )


def create_element_tree(element):
    """This creates an element tree, ready for writing to file.
    """
    return etree.ElementTree(element)


def generate_energy_values_element(energy_elements, energy_type):
    """This generates an EnergyValues element, containing the information given
    in the energy elements within a Collection element of the appropriate type.
    """
    energy_values = create_energy_values_element()
    if energy_type == "binding":
        energy_collection = generate_binding_energy_collection(energy_elements)
        energy_values.append(energy_collection)
        return energy_values
    elif energy_type == "free":
        energy_collection = generate_free_energy_collection(energy_elements)
        energy_values.append(energy_collection)
        return energy_values


def generate_free_energy_collection(free_energy_elements):
    """This generates a FreeEnergyCollection element, with the given BindingEnergy
    elements.
    """
    free_energy_collection = create_free_energy_collection_element()
    append_energies_to_collection(free_energy_collection, free_energy_elements)
    return free_energy_collection


def generate_binding_energy_collection(binding_energy_elements):
    """This generates a BindingEnergyCollection element, with the given BindingEnergy
    elements.
    """
    binding_energy_collection = create_binding_energy_collection_element()
    append_energies_to_collection(binding_energy_collection, binding_energy_elements)
    return binding_energy_collection


def append_energies_to_collection(energy_collection_element, energy_elements):
    """This appends the energies to the collection element.
    """
    for energy_element in energy_elements:
        energy_collection_element.append(copy.deepcopy(energy_element))


def create_energy_values_element():
    """This creates a new EnergyValues Element.
    """
    return etree.Element(PHASE + "EnergyValues", nsmap=PHASE_NAMESPACE_DICT)


def create_free_energy_collection_element():
    """This creates a FreeEnergyCollection element.
    """
    return etree.Element(PHASE + "FreeEnergyCollection", nsmap=PHASE_NAMESPACE_DICT)


def create_binding_energy_collection_element():
    """This creates a BindingEnergyCollection element.
    """
    return etree.Element(PHASE + "BindingEnergyCollection", nsmap=PHASE_NAMESPACE_DICT)


def get_energy_elements(element_tree, xpath_expression):
    """This gets Energy elements that match the given XPath expression.
    """
    energy_elements = element_tree.xpath(xpath_expression, namespaces=PHASE_NAMESPACE_DICT)
    return [copy.deepcopy(energy_element) for energy_element in energy_elements]


def generate_free_energy_xpath_expression(to_solvent_id):
    """This generates the XPath expression needed to get values for free energies.
    """
    return "/phase:EnergyValues/phase:FreeEnergyCollection/phase:FreeEnergy[@phase:fromSolventID=''][@phase:toSolventID='{:s}']".format(
        to_solvent_id
    )


def generate_binding_energy_xpath_expression(to_solvent_id):
    """This generates the XPath expression needed to get values for binding energies.
    """
    return "/phase:EnergyValues/phase:BindingEnergyCollection/phase:BindingEnergy[@phase:fromSolventID=''][@phase:toSolventID='{:s}']".format(
        to_solvent_id
    )


def read_xml_file(filename):
    """This reads in an xml file and returns ElementTree. Note that this is
    used to handle the large file size.
    """
    parser = etree.XMLParser(huge_tree=True)
    return etree.parse(filename, parser=parser)
