#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for reading in the values for the solvent maps, generating the
datapoints from the values read in.

@author: mark
"""

import logging
import phasexmlparser.parsers.solvationenergyparser as solvparser
from resultsanalysis.dataclasses.datapoints import Datapoints
from resultsanalysis.dataclasses.datapoint import Datapoint
from resultsanalysis.dataclasses.units import Units
from resultsanalysis.dataclasses.valuetype import ValueType
from resultsanalysis.dataclasses.value import Value

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def create_binding_datapoints_with_values(binding_energy_information_dict):
    """This creates a datapoints object for binding energy information.
    """
    datapoints = Datapoints()
    for key, info_dict in binding_energy_information_dict.items():
        datapoints.addDatapoint(create_binding_datapoint(info_dict))
    return datapoints


def create_free_energy_datapoints_with_values(free_energy_information_dict):
    """This creates a datapoints object for free energy information.
    """
    datapoints = Datapoints()
    for key, info_dict in free_energy_information_dict.items():
        datapoints.addDatapoint(create_free_energy_datapoint(info_dict))
    return datapoints


def create_binding_datapoint_list(binding_energy_information_dict):
    """This creates a datapoint for each piece of binding_energy_information, appending to list.
    """
    datapoint_list = []
    for key, info_dict in binding_energy_information_dict.items():
        datapoint_list.append(create_binding_datapoint(info_dict))
    return datapoint_list


def create_binding_datapoint(binding_energy_information):
    """This creates a datapoint for a binding energy point.
    """
    ssip_float_value = extract_ssip_value_from_solute_name(
        binding_energy_information["molecule_id"]
    )
    ssip_value = Value(
        ssip_float_value, r"\epsilon_{i}", ValueType.undefined, Units.dimensionless
    )
    solvation_energy_value = Value(
        binding_energy_information["total_energy"],
        r"\Delta G_{b,i}",
        ValueType.undefined,
        Units.kj_per_mol,
    )
    return Datapoint.from_values(
        ssip_value, solvation_energy_value, binding_energy_information["molecule_id"]
    )


def create_free_energy_datapoint(free_energy_information):
    """This creates a datapoint for a free energy point.
    """
    ssip_float_value = extract_ssip_value_from_solute_name(
        free_energy_information["molecule_id"]
    )
    ssip_value = Value(
        ssip_float_value, r"\epsilon_{i}", ValueType.undefined, Units.dimensionless
    )
    solvation_energy_value = Value(
        free_energy_information["total_energy"],
        r"\Delta G_{S,i}",
        ValueType.undefined,
        Units.kj_per_mol,
    )
    return Datapoint.from_values(
        ssip_value, solvation_energy_value, free_energy_information["molecule_id"]
    )


def extract_ssip_value_from_solute_name(solute_name):
    """This converts converts the solute name into a float.
    """
    return float(solute_name.replace("solute", ""))


def parse_binding_energy_info(filename):
    """This parses binding energy elements from the given file, producing a dictionary.
    """
    return solvparser.parse_solvation_binding_energy_info(filename)


def parse_free_energy_info(filename):
    """This parses binding energy elements from the given file, producing a dictionary.
    """
    return solvparser.parse_solvation_free_energy_info(filename)
