#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for generating the matrices required for creating dendrograms using set of value ranges.

@author: mark
"""

import logging
import numpy as np
import re
from scipy.spatial.distance import squareform
from matplotlib.ticker import PercentFormatter
import solventmapcreator.polynomialanalysis.polynomialcomparison as polycomp
import solventmapcreator.clusteringanalysis.clusteranalysis as cluster
import resultsanalysis.resultsoutput.plottinginput as plottinginput
import solventmapcreator.io.similaritymatrixwriter as matwrite

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)


def calculate_dendrogram_and_stats(
    x_values_list, polynomial_order, poly_data_by_id, filename_stem, **kwargs
):
    """This generates the reduced matrix, and then carries out dendrogram creation.
    """
    stacked_matrix, norm_coeffs = generate_normalised_rmsd_matrix_stack(
        x_values_list, polynomial_order, poly_data_by_id, **kwargs
    )
    mean_matrix = condense_matrix_to_mean(stacked_matrix)
    condensed_matrix = squareform(mean_matrix)
    labels = sorted(poly_data_by_id.keys())
    max_index = np.unravel_index(mean_matrix.argmax(), mean_matrix.shape)
    LOGGER.info("max value index: %s", max_index)
    LOGGER.info("max value: %f", mean_matrix[max_index[0], max_index[1]])
    LOGGER.info("solvents: %s %s", labels[max_index[0]], labels[max_index[1]])
    LOGGER.info("normalisation coefficients:")
    LOGGER.info(norm_coeffs)
    leaf_type = kwargs.pop("leaf_type", None)
    if "binary" == leaf_type:
        binary_pattern = re.compile(r"\d\.\d{3}")
        labels = [
            "{:.0f}".format(float(re.findall(binary_pattern, solv_id)[0]) * 100)
            for solv_id in labels
        ]
        LOGGER.info("new labels:")
        LOGGER.info(labels)
    dend_results = cluster.create_dend_and_stats_all_methods(
        condensed_matrix, labels, filename_stem, **kwargs
    )
    if kwargs.get("writematrix", False):
        matrix_out = matwrite.write_similarity_matrix_file(
            filename_stem + "_mat.csv", {"labels": labels, "value_matrix": mean_matrix}
        )
    else:
        matrix_out = None
    if kwargs.get("writestackmatrix", False):
        for domain in range(len(norm_coeffs)):
            LOGGER.debug("domain: %i", domain)
            matwrite.write_similarity_matrix_file(
            filename_stem + "_domain{}".format(domain) + "_mat.csv", {"labels": labels, "value_matrix": stacked_matrix[:,:,domain]}
        )
    if kwargs.get("plothist", False):
        hist_result = create_dist_histogram(
            condensed_matrix, filename_stem + "_hist", **kwargs
        )
        return dend_results, hist_result, matrix_out
    return dend_results, matrix_out


def create_dist_histogram(condensed_matrix, filename_stem, **kwargs):
    """This creates a distance histogram.
    """
    fig, axis = plottinginput.create_plot_with_axis(
        plottinginput.FIGURE_SIZES["vertical bar chart"]
    )
    number_of_distances = condensed_matrix.shape[0]
    LOGGER.debug("num distances: %i", number_of_distances)
    axis.hist(condensed_matrix, bins=kwargs.pop("binnumber", 20), density=True, color="k")
    axis.set_xlim([0.0, 1.0])
    axis.set_xlabel("SSI", fontsize=16)
    axis.set_ylabel("Percentage of distances", fontsize=16)
    axis.yaxis.set_major_formatter(PercentFormatter(xmax=100.0, decimals=0))
    axis.tick_params(axis='both', labelsize=kwargs.get("label_fontsize", 16))
    fileformat = kwargs.pop("fileformat", "svg")
    output_filename = filename_stem + "." + fileformat
    plottinginput.plt.savefig(output_filename, format=fileformat)
    plottinginput.plt.close(fig)


def condense_matrix_to_mean(stacked_matrix):
    """This takes the mean of the values, to get an average similarity
    """
    condensed_matrix = np.ndarray((0, len(stacked_matrix)))
    for row_i in range(stacked_matrix.shape[0]):
        mean_values = np.mean(stacked_matrix[row_i], axis=1).reshape(
            (1, len(stacked_matrix))
        )
        LOGGER.debug("mean_values:")
        LOGGER.debug(mean_values)
        condensed_matrix = np.append(condensed_matrix, mean_values, axis=0)
    return condensed_matrix


def generate_normalised_rmsd_matrix_stack(
    x_values_list, polynomial_order, poly_data_by_id, **kwargs
):
    """This generates the matrix of all the RMSD values.

    Normalised RMSD values between i and j curves are in the [i][j][:] slice.
    """
    stacked_matrix = np.empty((len(poly_data_by_id), len(poly_data_by_id), 0))
    LOGGER.debug("stacked matrix")
    LOGGER.debug(np.empty((len(poly_data_by_id), len(poly_data_by_id), 0)))
    norm_coefficients = []
    normalisation_factors = kwargs.get("norm_coefficients", None)
    LOGGER.info("Calculating RMSD")
    for index, x_values in enumerate(x_values_list):
        LOGGER.debug("domain: %i", index + 1)
        if normalisation_factors:
            norm_factor = normalisation_factors[index]
        else:
            norm_factor = None
        norm_matrix, norm_coeff = generate_normalised_rmsd_matrix(
            x_values, polynomial_order, poly_data_by_id, norm_coefficient=norm_factor
        )
        norm_matrix = norm_matrix.reshape(
            (len(poly_data_by_id), len(poly_data_by_id), 1)
        )
        LOGGER.debug("norm matrix")
        LOGGER.debug(norm_matrix)
        stacked_matrix = np.append(stacked_matrix, norm_matrix, axis=-1)
        norm_coefficients.append(norm_coeff)
    return stacked_matrix, norm_coefficients


def generate_normalised_rmsd_matrix(
    x_values, polynomial_order, poly_data_by_id, **kwargs
):
    """This generates a normalised matrix. Where the values are scaled relative
    to the largest value in the matrix.
    """
    rmsd_matrix = polycomp.calculate_rmsd_matrix_from_poly_coeff(
        x_values, polynomial_order, poly_data_by_id
    )
    if kwargs.get("norm_coefficient", None):
        normalisation_coeff = kwargs.get("norm_coefficient", None)
        return rmsd_matrix / normalisation_coeff, normalisation_coeff
    if len(poly_data_by_id) <= 1:
        normalisation_coeff = 1.0
    else:
        normalisation_coeff = rmsd_matrix.max()
        LOGGER.debug("calculated norm coeff: %f", normalisation_coeff)
        norm_index = np.unravel_index(rmsd_matrix.argmax(),rmsd_matrix.shape)
        LOGGER.debug("norm indices: %s", norm_index)
        poly_ids = sorted(poly_data_by_id.keys())
        LOGGER.debug("Solvent 1: %s", poly_ids[norm_index[0]])
        LOGGER.debug("Solvent 2: %s", poly_ids[norm_index[1]])
    return rmsd_matrix / normalisation_coeff, normalisation_coeff
