#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for analysing the polynomial fitting data.

This will provide tools to analyse which order gives the best fit to the data.

@author: mark
"""

import logging
import numpy as np

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def get_pos_neg_rmse_by_order_asarrays(polynomial_dict_by_solvent_id, order_list):
    """This gets the mean RMSE by order and returns it as an (2,n) array for
    the positive and negative RMSE fits.
    """
    pos_neg_dict = get_pos_neg_rmse_by_order(polynomial_dict_by_solvent_id, order_list)
    pos_neg_array_dict = {}
    for key in pos_neg_dict:
        mean_rmse_array = np.empty((2, len(order_list)))
        for i in range(len(order_list)):
            order = order_list[i]
            mean_rmse_array[0][i] = order
            mean_rmse_array[1][i] = pos_neg_dict[key][order]
        pos_neg_array_dict[key] = mean_rmse_array
    return pos_neg_array_dict


def get_mean_rmse_by_order_asarray(polynomial_dict_by_solvent_id, order_list):
    """This gets the mean RMSE by order and returns it as an (2,n) array.
    """
    mean_rmse_dict = get_mean_rmse_by_order(polynomial_dict_by_solvent_id, order_list)
    mean_rmse_array = np.empty((2, len(order_list)))
    for i in range(len(order_list)):
        order = order_list[i]
        mean_rmse_array[0][i] = order
        mean_rmse_array[1][i] = mean_rmse_dict[order]
    return mean_rmse_array


def get_pos_neg_rmse_by_order(polynomial_dict_by_solvent_id, order_list):
    """This gets the mean RMSE by order for the positive and negative data.
    """
    mean_pos_rmse_dict = {}
    mean_neg_rmse_dict = {}
    for order in order_list:
        mean_pos_rmse_dict[order] = get_mean_positive_rmse(
            polynomial_dict_by_solvent_id, order
        )
        mean_neg_rmse_dict[order] = get_mean_negative_rmse(
            polynomial_dict_by_solvent_id, order
        )
    return {"positive": mean_pos_rmse_dict, "negative": mean_neg_rmse_dict}


def get_mean_rmse_by_order(polynomial_dict_by_solvent_id, order_list):
    """This gets the mean RMSE by order.
    """
    mean_rmse_dict = {}
    for order in order_list:
        mean_rmse_dict[order] = get_mean_rmse(polynomial_dict_by_solvent_id, order)
    return mean_rmse_dict


def get_mean_negative_rmse(polynomial_dict_by_solvent_id, order):
    """This extracts the rmse values to an array for the positive value fit,
    then calculates the mean.
    """
    rmse_array = np.array(
        [
            polynomial_dict_by_solvent_id[solvent_id][order]["negative"]["RMSE"]
            for solvent_id in polynomial_dict_by_solvent_id.keys()
        ]
    )
    return rmse_array.mean()


def get_mean_positive_rmse(polynomial_dict_by_solvent_id, order):
    """This extracts the rmse values to an array for the positive value fit,
    then calculates the mean.
    """
    rmse_array = np.array(
        [
            polynomial_dict_by_solvent_id[solvent_id][order]["positive"]["RMSE"]
            for solvent_id in polynomial_dict_by_solvent_id.keys()
        ]
    )
    return rmse_array.mean()


def get_mean_rmse(polynomial_dict_by_solvent_id, order):
    """This extracts the rmse values to an array, then calculates the mean.
    """
    rmse_array = np.array(
        [
            polynomial_dict_by_solvent_id[solvent_id][order]["RMSE"]
            for solvent_id in polynomial_dict_by_solvent_id.keys()
        ]
    )
    return rmse_array.mean()


def get_total_sorted_by_alt_metric(full_order_list, best_order_sorted_dict):
    """This gets the sorted information and orders them ready to plot them.
    Need to feed in a list of all the orders, as not all may appear.
    """
    numbers_by_order = []
    for i in range(len(full_order_list)):
        order = full_order_list[i]
        order_info = []
        for j in range(len(full_order_list)):
            sorted_dict_order = full_order_list[j]
            if sorted_dict_order in best_order_sorted_dict.keys():
                if order in best_order_sorted_dict[sorted_dict_order].keys():
                    order_info.append(
                        len(best_order_sorted_dict[sorted_dict_order][order])
                    )
                else:
                    order_info.append(0)
            else:
                order_info.append(0)
        numbers_by_order.append(tuple(order_info))
    numbers_by_order_tuple = tuple(numbers_by_order)
    return (tuple(full_order_list), numbers_by_order_tuple)


def get_total_by_fit_order(best_order_dict):
    """This returns a tuple containing the order, and also the number of
    occurances where the corresponding order provided the best fit.
    """
    order_tuple = tuple(sorted(best_order_dict.keys()))
    number_by_order = tuple([len(best_order_dict[order]) for order in order_tuple])
    return (order_tuple, number_by_order)


def generate_lists_by_alt_metric(
    full_order_list, solvent_ids_by_order, solvent_ids_by_order_alt_metric
):
    """This goes through each order in the data and uses the best polynomial
    fit data to sort the solvent_ids by the alternative metric.
    """
    solvent_ids_sorted_by_alt_metric = {}
    for order, solvent_id_list in solvent_ids_by_order.items():
        solvent_ids_sorted_by_alt_metric[order] = extract_lists_by_order_of_alt_metric(
            full_order_list, solvent_id_list, solvent_ids_by_order_alt_metric
        )
    return solvent_ids_sorted_by_alt_metric


def extract_lists_by_order_of_alt_metric(
    full_order_list, solvent_id_list, solvent_ids_by_order
):
    """This returns a dictionary where the solvent IDs are sorted based on the
    best order from the alternate metric.
    """
    solvent_id_alt_metric = {}
    for order in full_order_list:
        metric_ids_for_order = []
        if order in solvent_ids_by_order.keys():
            for solvent_id in solvent_id_list:
                if solvent_id in solvent_ids_by_order[order]:
                    metric_ids_for_order.append(solvent_id)
            solvent_id_alt_metric[order] = metric_ids_for_order
    return solvent_id_alt_metric


def get_solvent_id_by_order(best_order_dict):
    """This gets the values from the best order dict and returns lists of
    solvent IDs group by the order of the polynomial that gives the best fit.
    """
    solvent_ids_by_order = {}
    for order in best_order_dict.keys():
        solvent_ids_by_order[order] = get_solvent_id(best_order_dict[order])
    return solvent_ids_by_order


def get_solvent_id(order_info_list):
    """This gets the values from the order info list about which solvents are in this set.
    """
    return [order_info_list[i][0] for i in range(len(order_info_list))]


def get_smallest_order_below_threshold(polynomial_dict_by_solvent_id, threshold):
    """This returns dict with tuples containing the information.
    """
    smallest_rmse_order_dict = {}
    for solvent_id, polynomial_dict in polynomial_dict_by_solvent_id.items():
        smallest_order, rmse_value = extract_smallest_order_below_threshold(
            polynomial_dict, threshold
        )
        info_tuple = (solvent_id, rmse_value)
        if smallest_order not in smallest_rmse_order_dict.keys():
            smallest_rmse_order_dict[smallest_order] = [info_tuple]
        else:
            smallest_rmse_order_dict[smallest_order].append(info_tuple)
    return smallest_rmse_order_dict


def extract_smallest_order_below_threshold(polynomial_dict, threshold):
    """This gets the order of the polynomial below which the RMSE drops below
    the threshold given.
    """
    smallest_order = None
    rmse_value = None
    for order, info in polynomial_dict.items():
        if smallest_order is None and info["RMSE"] < threshold:
            smallest_order = order
            rmse_value = info["RMSE"]
        elif info["RMSE"] < threshold and order < smallest_order:
            smallest_order = order
            rmse_value = info["RMSE"]
    return smallest_order, rmse_value


def get_best_poly_fit_rmse(polynomial_dict_by_solvent_id):
    """This returns a dict with tuples containing the information.
    """
    best_rmse_order_dict = {}
    for solvent_id, polynomial_dict in polynomial_dict_by_solvent_id.items():
        best_order, smallest_rmse = extract_best_poly_fit_rmse(polynomial_dict)
        info_tuple = (solvent_id, smallest_rmse)
        if best_order not in best_rmse_order_dict.keys():
            best_rmse_order_dict[best_order] = [info_tuple]
        else:
            best_rmse_order_dict[best_order].append(info_tuple)
    return best_rmse_order_dict


def extract_best_poly_fit_rmse(polynomial_dict):
    """This returns the order with the lowest value of RMSE,
    and value of the RMSE.
    """
    best_order = None
    smallest_rmse = None
    for order, info in polynomial_dict.items():
        if smallest_rmse is None:
            smallest_rmse = info["RMSE"]
            best_order = order
        elif smallest_rmse > info["RMSE"]:
            smallest_rmse = info["RMSE"]
            best_order = order
    return best_order, smallest_rmse


def get_best_poly_fit_covar(polynomial_dict_by_solvent_id):
    """This returns a dict with tuples containing the information.
    """
    best_covar_order_dict = {}
    for solvent_id, polynomial_dict in polynomial_dict_by_solvent_id.items():
        best_order, smallest_covar = extract_best_poly_fit_covar(polynomial_dict)
        info_tuple = (solvent_id, smallest_covar)
        if best_order not in best_covar_order_dict.keys():
            best_covar_order_dict[best_order] = [info_tuple]
        else:
            best_covar_order_dict[best_order].append(info_tuple)
    return best_covar_order_dict


def extract_best_poly_fit_covar(polynomial_dict):
    """This returns the order with the lowest covariance, and the
    value of covariance.
    """
    best_order = None
    smallest_covar = None
    for order, info in polynomial_dict.items():
        if smallest_covar is None:
            smallest_covar = info["covar"]
            best_order = order
        elif smallest_covar > info["covar"]:
            smallest_covar = info["covar"]
            best_order = order
    return best_order, smallest_covar
