#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for taking the datapoints for the solvation curve, and generating
the polynomial fit data and also the plots.

@author: mark
"""

import logging
import numpy as np
import solventmapcreator.polynomialanalysis.polynomialvaluecalculator as polyvalcalc
import resultsanalysis.resultsoutput.plottinginput as plottinginput
import solventmapcreator.io.polynomialdatawriter as polynomialdatawriter
import solventmapcreator.io.solvationenergyreader as solvationenergyreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)


def plot_free_energies_from_file(free_energy_filename, polynomial_order, **kwargs):
    """This creates in the plotting information, and outputs a plot to file.
    """
    plot_data = parse_free_energies_create_plot_input_data(
        free_energy_filename,
        polynomial_order,
        kwargs.get("pos_subset_list", None),
        kwargs.get("neg_subset_list", None),
    )
    if type(plot_data["polynomial coefficients"]) is dict:
        return create_scatter_with_split_poly(plot_data, **kwargs)
    else:
        return plottinginput.create_scatter_graph_with_polynomial(plot_data)


def plot_binding_energies_from_file(
    binding_energy_filename, polynomial_order, **kwargs
):
    """This creates in the plotting information, and outputs a plot to file.
    """
    plot_data = parse_binding_energies_create_plot_input_data(
        binding_energy_filename,
        polynomial_order,
        kwargs.get("pos_subset_list", None),
        kwargs.get("neg_subset_list", None),
    )
    if type(plot_data["polynomial coefficients"]) is dict:
        return create_scatter_with_split_poly(plot_data, **kwargs)
    else:
        return plottinginput.create_scatter_graph_with_polynomial(plot_data)


def create_scatter_with_split_poly(input_data, **kwargs):
    """This generates the split plot when a split polynomial is required.
    """
    scatter_plot, scatter_plot_axis = plottinginput.plot_scatter_graph(input_data)
    x_data = np.sort(input_data["x_data"])
    LOGGER.debug("X data shape: , X data:")
    LOGGER.debug(x_data.shape)
    LOGGER.debug(x_data)
    polynomial_values = polyvalcalc.calculate_polynomial_values(
        x_data, input_data["polynomial coefficients"]
    )
    plottinginput.plot_polynomial_curve(
        scatter_plot_axis, x_data, polynomial_values, input_data["polynomial order"]
    )
    fileformat = kwargs.get("fileformat", "eps")
    output_filename = plottinginput.create_output_filename(input_data, fileformat)
    plottinginput.plt.savefig(output_filename, format=fileformat)
    plottinginput.plt.close(scatter_plot)
    return 0


def parse_free_energies_create_plot_input_data(
    free_energy_filename, polynomial_order, pos_subset_list=None, neg_subset_list=None
):
    """This creates the input data for a plot. This overrides the default figure
    label, so that the figures can be distinguished based on the solvent.
    """
    datapoints = parse_free_energy_from_file_with_data_arrays(free_energy_filename)
    plot_data = datapoints.generateScatterPlotParameters(
        polynomial_order=polynomial_order
    )
    if pos_subset_list is not None and neg_subset_list is not None:
        poly_coeff_dict = generate_poly_pos_neg_separate_fit(
            datapoints, polynomial_order, pos_subset_list, neg_subset_list
        )
        plot_data["polynomial coefficients"] = poly_coeff_dict
    plot_range = plot_data["plot_axis_range"]
    plot_range[3] = plot_range[3] + 1.0
    plot_data["plot_axis_range"] = plot_range
    plot_data["figure_label"] = free_energy_filename.replace(".xml", "_fe")
    return plot_data


def parse_binding_energies_create_plot_input_data(
    binding_energy_filename,
    polynomial_order,
    pos_subset_list=None,
    neg_subset_list=None,
):
    """This creates the input data for a plot. This overrides the default figure
    label, so that the figures can be distinguished based on the solvent.
    """
    datapoints = parse_binding_energy_from_file_with_data_arrays(
        binding_energy_filename
    )
    plot_data = datapoints.generateScatterPlotParameters(
        polynomial_order=polynomial_order
    )
    if pos_subset_list is not None and neg_subset_list is not None:
        poly_coeff_dict = generate_poly_pos_neg_separate_fit(
            datapoints, polynomial_order, pos_subset_list, neg_subset_list
        )
        plot_data["polynomial coefficients"] = poly_coeff_dict
    plot_range = plot_data["plot_axis_range"]
    plot_range[3] = plot_range[3] + 1.0
    plot_data["plot_axis_range"] = plot_range
    plot_data["figure_label"] = binding_energy_filename.replace(".xml", "_be")
    return plot_data


def generate_poly_pos_neg_separate_fit(
    datapoints, order, pos_subset_list, neg_subset_list
):
    """This generates the fit for the positive and negative datapoints separately.
    A dictionary containing the 2 fits are returned.
    """
    poly_dict = polynomialdatawriter.generate_polynomial_fit_data_positive_negative_separate_fit(
        datapoints, order, pos_subset_list, neg_subset_list
    )
    return {
        "positive": poly_dict["positive"]["coefficients"],
        "negative": poly_dict["negative"]["coefficients"],
    }


def parse_free_energy_poly_data_to_file_split_fit(
    free_energy_filename, order_list, poly_filename, pos_subset_list, neg_subset_list
):
    """This reads in the binding energy information, and then performs the fits on the
    two separate subsets, and outputs to file.
    """
    datapoints = parse_free_energy_from_file_with_data_arrays(free_energy_filename)
    return write_poly_data_to_file_split_fit(
        datapoints, order_list, poly_filename, pos_subset_list, neg_subset_list
    )


def parse_binding_energy_poly_data_to_file_split_fit(
    binding_energy_filename, order_list, poly_filename, pos_subset_list, neg_subset_list
):
    """This reads in the binding energy information, and then performs the fits on the
    two separate subsets, and outputs to file.
    """
    datapoints = parse_binding_energy_from_file_with_data_arrays(
        binding_energy_filename
    )
    return write_poly_data_to_file_split_fit(
        datapoints, order_list, poly_filename, pos_subset_list, neg_subset_list
    )


def write_poly_data_to_file_split_fit(
    datapoints, order_list, filename, pos_subset_list, neg_subset_list
):
    """This calculates the polynomial information for fitting to the 2 sub sets
    and writes to file.
    """
    return polynomialdatawriter.write_poly_data_to_file_split_fit(
        datapoints, order_list, filename, pos_subset_list, neg_subset_list
    )


def parse_free_energies_and_output_poly_data_subset(
    free_energy_filename, order_list, poly_filename, subset_list
):
    """This reads in the energy information, and then selects teh subset of
    points to carry out the analysis on, outputs the polynomial information to file.
    """
    datapoints = parse_free_energy_from_file_with_data_arrays(free_energy_filename)
    datapoints_subset = datapoints.createDatapointsSubGroup(subset_list)
    return write_poly_data_to_file(datapoints_subset, order_list, poly_filename)


def parse_binding_energies_and_output_poly_data_subset(
    binding_energy_filename, order_list, poly_filename, subset_list
):
    """This reads in the energy information, and then selects teh subset of
    points to carry out the analysis on, outputs the polynomial information to file.
    """
    datapoints = parse_binding_energy_from_file_with_data_arrays(
        binding_energy_filename
    )
    datapoints_subset = datapoints.createDatapointsSubGroup(subset_list)
    return write_poly_data_to_file(datapoints_subset, order_list, poly_filename)


def parse_free_energies_and_output_poly_data(
    free_energy_filename, order_list, poly_filename
):
    """This reads in the energy information, and then outputs the polynomial
    information to file.
    """
    datapoints = parse_free_energy_from_file_with_data_arrays(free_energy_filename)
    return write_poly_data_to_file(datapoints, order_list, poly_filename)


def parse_binding_energies_and_output_poly_data(
    binding_energy_filename, order_list, poly_filename
):
    """This reads in the energy information, and then outputs the polynomial
    information to file.
    """
    datapoints = parse_binding_energy_from_file_with_data_arrays(
        binding_energy_filename
    )
    return write_poly_data_to_file(datapoints, order_list, poly_filename)


def write_poly_data_to_file(datapoints, order_list, filename):
    """This calculates the polynomial information, and then outputs the
    information to file.
    """
    return polynomialdatawriter.write_poly_data_to_file(
        datapoints, order_list, filename
    )


def parse_free_energy_from_file_with_data_arrays(filename):
    """This reads in the information form file to a Datapoints object, and
    creates the data arrays.
    """
    return solvationenergyreader.parse_free_energy_from_file(filename)


def parse_binding_energy_from_file_with_data_arrays(filename):
    """This reads in the information form file to a Datapoints object, and
    creates the data arrays.
    """
    return solvationenergyreader.parse_binding_energy_from_file(filename)
