#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for calculating the Energy values when given polynomial coefficients.

This copes with fitting to positive and negative plot regions as well as
fitting over the full range.

@author: mark
"""

import logging
import numpy as np
import numpy.polynomial.polynomial as poly

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def calculate_polynomial_values(x_values, polynomial_coefficients):
    """This evaluates the polynomial function for the given x values.
    If a dict is given, signifying 2 different fitting regimes, that is used,
    else it just calls polyval, assuming a single numpy array of coefficients
    was given.
    """
    if type(polynomial_coefficients) is dict:
        if type(x_values) is float or type(x_values) is np.float64:
            return calculate_poly_value_from_dict(x_values, polynomial_coefficients)
        else:
            return calculate_poly_value_for_array_from_dict(
                x_values, polynomial_coefficients
            )

    else:
        return calculate_polynomial_value(x_values, polynomial_coefficients)


def calculate_poly_value_for_array_from_dict(x_values, poly_coeffs):
    """This calculates the polynomial values for the given x values,
    using a different coefficient set depending on x value sign.
    """
    poly_values = np.empty(x_values.shape)
    for i in range(len(x_values)):
        poly_values[i] = calculate_poly_value_from_dict(x_values[i], poly_coeffs)
    return poly_values


def calculate_poly_value_from_dict(x_value, poly_coeffs):
    """This calculates the polynomial value
    """
    if x_value < 0.0:
        return calculate_polynomial_value(x_value, poly_coeffs["negative"])
    else:
        return calculate_polynomial_value(x_value, poly_coeffs["positive"])


def calculate_polynomial_value(x_value, polynomial_coefficients):
    """This calculates the value.
    """
    return poly.polyval(x_value, polynomial_coefficients)
