#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for merging the molecules to the solvent map to create the functional
group interaction profile.

@author: mark
"""

import logging
import os
import re
import svgutils.transform as svgt

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

FGIP_MOLECULES_FILENAME = os.path.join(
    os.path.dirname(__file__), "resources/FGIPmolecules.svg"
)


def create_fgip_from_map(map_filename, **kwargs):
    """This creates a FGIP from the given solvent map.
    """
    svg_figure = create_svgfigure()
    map_plot = get_svg_plot(map_filename)
    map_plot.moveto(0.0, 95.0, scale_x=1.25)
    append_svg_plots(svg_figure, map_plot)
    output_filename = create_output_filename(map_filename)
    save_svg_figure(svg_figure, output_filename)
    latex_block = make_latex_text_block(map_filename, **kwargs)
    return latex_block


def make_latex_text_block(map_filename, **kwargs):
    """This produces a standard block of latex text for the figure eps version of the FGIP.
    """
    directory = kwargs.pop("directory", "solventmaps/")
    if directory != "":
        directory = directory +"/" if directory[-1] != "/" else directory
    output_filename = (
        create_output_filename(map_filename)
        .replace(".svg", ".eps")
    )
    molecule_name = (
        map_filename.replace("_solv_map.svg", "")
        .replace(directory, "")
        .replace("_", ",")
    )
    if kwargs.pop("solvent_type", None) == "binary":
        binary_pattern = re.compile(r"\d\.\d{3}")
        solv1frac = re.findall(binary_pattern, molecule_name)[0]
        solv1frac_percent = float(solv1frac) * 100
        molecules = molecule_name.split(solv1frac)
        molecule_name = "{0:.1f}\\% {1} {2:.1f}\\% {3}".format(
            solv1frac_percent, molecules[0], 100.0 - solv1frac_percent, molecules[1]
        )
    latex_block = """
\\begin{figure}[H]
    \\centering
    \\includegraphics[width=0.95\\textwidth]{%s}
    \\caption{FGIP for %s at 298K.}
    \\label{fig:%ssolvationmap}
\\end{figure}
""" % (
        output_filename,
        molecule_name,
        molecule_name.replace(" ", "")
        .replace(".", "_")
        .replace("(", "")
        .replace(")", "")
        .replace(",", "_")
        .replace("%", "")
        .replace("\\", ""),
    )
    return latex_block


def save_svg_figure(svg_figure, filename):
    """This saves the figue to file.
    """
    svg_figure.save(filename)


def create_output_filename(filename, suffix="_solv_map.svg"):
    """This returns the output filename to be used.
    """
    outfilename = (
        filename.replace(suffix, "")
        .replace(" ", "")
        .replace(".", "_")
        .replace("(", "")
        .replace(")", "")
    )
    outfilename += "_FGIP.svg"
    return outfilename


def append_svg_plots(svg_figure, svg_plot):
    """This appends the plot to the figure.
    """
    svg_figure.append(svg_plot)


def get_svg_plot(svg_filename):
    """This gets the plot elements.
    """
    figure = svgt.fromfile(svg_filename)
    return figure.getroot()


def create_svgfigure():
    """This creates SVG figure using the molecules as a base.
    """
    return svgt.fromfile(FGIP_MOLECULES_FILENAME)
