#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
This calculates the fractional occupancy for a phase. This calculates the
solvent contribution from the given file containing solvent xml, and adds the
contribution for the single solute SSIP at the given concentration.

@author: mark
"""

import logging
import phasexmlparser.parsers.fractionaloccupancyparser as fracparser

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def calculate_fractional_occupancy_for_phase(
    frac_occ_inf_dict, solvent_id, temperature
):
    """Extract Fractional Occupancy of appropriate phase from dictionary.

    Parameters
    ----------
    frac_occ_inf_dict : dict
        fractional occupancies labelled by solventID and temperature.
    solvent_id : str
        solvent ID.
    temperature : float
        temeprature in Kelvin.

    Returns
    -------
    fractional_occupancy : float
        fractional occupancy.

    """
    occ_id = solvent_id + "{:.3f}".format(temperature)
    LOGGER.debug("Occupancy ID: %s", occ_id)
    fractional_occupancy = frac_occ_inf_dict[occ_id]
    return fractional_occupancy


def read_fractional_occupancy_information(frac_occ_filename):
    """Read in the fractional occupancy information to a dictionary.
    
    The dictionary keys consist of solventID followed by temperature to 3dp.

    Parameters
    ----------
    frac_occ_filename : str
        File conatining fractional occupancy XML.

    Returns
    -------
    dict
        condensed dictionary of information labelled by solvent ID and temperature.

    """
    return fracparser.parse_fractional_occupancy_file(frac_occ_filename)