#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for calculating gamma term for a solvent.


@author: mdd31
"""

import logging
import numpy as np
import phasexmlparser.parsers.associationenergyparser as assocparser
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator
import solventmapcreator.solvationcalculation.solvationcalculator as solvcalculator
import solventmapcreator.solvationcalculation.solvationmapgenerator as solvmapgenerator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

# epsilon and temperature values are same for all, so can be set with dictionary.

GAMMA_INFO = {
    "epsilon_i": -10.0,
    "epsilon_j": 5.0,
    "temperature": 298.0,
    "solute_concentration": 0.001,
}

# Display of results and summary information
# process dicts of solvent_id:gamma value to bar plots and other statistics.


def calculate_mean_and_sigma(gamma_data_dict):
    """This calculates the mean and standard deviation for the data set.
    """
    data = np.array(list(gamma_data_dict.values()))
    return np.mean(data), np.std(data)


# Simulation Calculation


def calculate_simulation_gamma_for_solvent_list(
    gamma_info,
    solvent_id_list,
    solvent_file_tree,
    assoc_info,
    directory,
    polynomial_order,
):
    """This calculates the gamma from the simulation for all solvents in list.
    """
    gamma_information = {}
    for solvent_id in solvent_id_list:
        calculated_gamma = calculate_simulation_gamma(
            gamma_info,
            solvent_id,
            solvent_file_tree,
            assoc_info,
            directory,
            polynomial_order,
        )
        gamma_information[solvent_id] = calculated_gamma
    return gamma_information


def calculate_simulation_gamma(
    gamma_info, solvent_id, solvent_file_tree, assoc_info, directory, polynomial_order
):
    """This is the difference between the calculated delta delta G_Hbonds and
    the free energy value from the simulation.
    """
    theta = calculate_fractional_occupancy_for_phase(
        solvent_file_tree, solvent_id, gamma_info["temperature"]
    )
    poly_filename = create_poly_filename_from_solvent_id(directory, solvent_id)
    poly_coeffs = parse_polynomial_data_file_extract_polynomial_coefficients(
        poly_filename, polynomial_order
    )
    calc_assoc_energy = calculate_association_energy(
        gamma_info["epsilon_i"],
        gamma_info["epsilon_j"],
        gamma_info["temperature"],
        theta,
        poly_coeffs,
    )
    sim_assoc_energy = assoc_info[solvent_id]["value"]
    return sim_assoc_energy - calc_assoc_energy


def calculate_association_energy(
    epsilon_i, epsilon_j, temperature, theta, polynomial_coefficients
):
    """This calculates the association energy
    """
    return solvcalculator.calculate_association_energy(
        epsilon_i, epsilon_j, temperature, theta, polynomial_coefficients
    )


# Analytical calculation


def calculate_analytical_gamma_for_solvent_list(
    epsilon1,
    epsilon2,
    temperature,
    frac_occ_inf_dict,
    solvent_id_list,
):
    """This calculates the analytical gamma for the solvents.
    """
    gamma_information = {}
    for solvent_id in solvent_id_list:
        calculated_gamma = calculate_analytical_gamma_for_solvent(
            epsilon1,
            epsilon2,
            temperature,
            frac_occ_inf_dict,
            solvent_id,
        )
        gamma_information[solvent_id] = calculated_gamma
    return gamma_information


def calculate_analytical_gamma_for_solvent(
    epsilon1, epsilon2, temperature, frac_occ_inf_dict, solvent_id
):
    """This calculates the analytical gamma for the solvent.
    """
    theta = calculate_fractional_occupancy_for_phase(
        frac_occ_inf_dict, solvent_id, temperature
    )
    return calculate_analytical_gamma(epsilon1, epsilon2, theta, temperature)


def calculate_analytical_gamma(epsilon1, epsilon2, theta, temperature):
    """This calculates the analytical gamma value for the solvent information,
    based on the tight binding limit.
    """
    assoc_energy_term = (
        solvcalculator.GAS_CONSTANT
        * solvcalculator.CONVERSION_FROM_J_TO_KJ
        * temperature
        * np.log(0.5 * theta * solvcalculator.CMAX)
    )
    return assoc_energy_term


# input file parsing


def parse_elements_from_file(filename):
    """This parses the association information from file.
    """
    assoc_info_list = assocparser.parse_elements_from_file(filename, None)
    assoc_dict = {}
    for assoc_info in assoc_info_list:
        assoc_dict[assoc_info["solvent_id"]] = assoc_info
    return assoc_dict


def create_poly_filename_from_solvent_id(directory, solvent_id):
    """This creates the filename from directory and solvent_id information.
    """
    if directory is not None:
        return (
            directory
            + "/"
            + solvent_id.replace(",", "_")
            + "binding_poly_fit_split.csv"
        )
    else:
        return solvent_id.replace(",", "_") + "binding_poly_fit_split.csv"


def parse_polynomial_data_file_extract_polynomial_coefficients(
    filename, polynomial_order
):
    """This parses a polynomial file and gets the coefficients for the given
    polynomial order.
    """
    return solvmapgenerator.parse_polynomial_data_file_extract_polynomial_coefficients(
        filename, polynomial_order
    )


def calculate_fractional_occupancy_for_phase(
    frac_occ_inf_dict, solvent_id, temperature
):
    """Extract Fractional Occupancy of appropriate phase from dictionary.

    Parameters
    ----------
    frac_occ_inf_dict : dict
        fractional occupancies labelled by solventID and temperature.
    solvent_id : str
        solvent ID.
    temperature : float
        temeprature in Kelvin.

    Returns
    -------
    fractional_occupancy : float
        fractional occupancy.

    """
    return fractionaloccupancycalculator.calculate_fractional_occupancy_for_phase(
        frac_occ_inf_dict, solvent_id, temperature
    )

