# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Interaction energy calculator class.

Handles temperature dependence of interaction between two SSIPs. This is a
transpliation of the implementation in the Java code, with some modifications
to behave in a more pythonic manner.

@author: mark
"""

import logging
import numpy as np
import math
import scipy.interpolate as interpolate

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

"""
Value for the gas constant, R in kJK^-1mol^-1.
https://en.wikipedia.org/wiki/Gas_constant
"""
GAS_CONSTANT = 8.3144598E-3

"""Room temperature in Kelvin."""
ROOM_TEMPERATURE = 298.0

class InteractionEnergyCalculator(object):
    """Class containing methods for interaction energy calculation."""

    interaction_energy_spline = None

    @staticmethod
    def calculate_interaction_energy(e0, temperature):
        """Calculate interaction energy.

        Parameters
        ----------
        e0 : float
            Energy of interaction at 0K.
        temperature : float
            Temperature in Kelvin.

        Returns
        -------
        float
            Interaction energy at given temperature.

        """
        interactionenergy_func = InteractionEnergyCalculator.interaction_energy_function()
        return interactionenergy_func(e0, temperature)
    @staticmethod
    def calculate_interaction_energy_rt(e0):
        """Calculate interaction energy at room temperature (298K).

        Parameters
        ----------
        e0 : float
            Interaction energy at 0K.

        Returns
        -------
        float
            Interaction energy at room temperature.

        """
        interactionenergy_func_rt = InteractionEnergyCalculator.interaction_energy_function_rt()
        return interactionenergy_func_rt(e0)
    @classmethod
    def calculate_interaction_energy_using_spline(cls, e298, temperature):
        """Calculate interaction energy using spline fit.
        
        Interaction Energy at given temperature is calculated by extrapolation
        of interaction at 298K to interaction energy at 0K using a cubic spline
        in the mapping. The energy at the given temperature is then calculated.
        

        Parameters
        ----------
        cls : InteractionEnergyCalculator
        e298 : float
            Interaction energy at 298K
        temperature : float
            Temperature.

        Returns
        -------
        float
            Interaction energy at given temperature

        """
        interaction_energy_az = cls.calculate_interaction_energy_az(e298)
        return cls.calculate_interaction_energy(interaction_energy_az, temperature)
    @classmethod
    def calculate_interaction_energy_az(cls, e298):
        """Calculate interaction energy at 0K.

        Parameters
        ----------
        cls : InteractionEnergyCalculator
        e298 : float
            Interaction energy at 298K.

        Returns
        -------
        float
            Interaction energy at 0K.

        """
        if cls.interaction_energy_spline is None:
            cls.interaction_energy_spline = cls.calculate_interaction_energy_spline()
        return cls.interaction_energy_spline(e298)
    @staticmethod
    def calculate_interaction_energy_spline():
        """Calculate interaction energy spline.

        Returns
        -------
        interp1d fit
            Cubic spline fit.

        """
        interaction_energies_for_fit = InteractionEnergyCalculator.interaction_energy_values_for_fit()
        interaction_energies_298k = InteractionEnergyCalculator.interaction_energy_array_rt(interaction_energies_for_fit)
        # Want the spline function to return value of E0 s we use the
        # interaction energies at 298K as x values and E0 as y values.
        return interpolate.interp1d(interaction_energies_298k, interaction_energies_for_fit,kind='cubic')
    @staticmethod
    def interaction_energy_array_rt(e0_array):
        """Calculate interaction energy at 298K for array.

        Parameters
        ----------
        e0_array : np.array
            Array of interaction energies at 0K

        Returns
        -------
        np.array
            Array of interaction energies at 298K.

        """
        vectored_ie_rt = np.vectorize(InteractionEnergyCalculator.interaction_energy_function_rt())
        return vectored_ie_rt(e0_array)
    @staticmethod
    def interaction_energy_values_for_fit():
        """Interaction energy values at 0K between -400.0 and 0.0 in 0.05 intervals.

        Returns
        -------
        Array
            Array of ineraction energies.

        """
        return np.arange(-400.0, 0.1, 0.05)
    @staticmethod
    def interaction_energy_function():
        """Interaction energy function.
        
        This returns a function to calculate the interaction energy. Requires
        energy at 0K and temperature.
        

        Returns
        -------
        function
            Interaction energy calculator, accepts e0 and temperature.

        """
        return lambda e0, temperature : e0 /( 1 + math.exp(e0/(GAS_CONSTANT * temperature)))
    @staticmethod
    def interaction_energy_function_rt():
        """Interaction energy function at room temperature.

        Returns
        -------
        function
            Interaction energy at 298K calculator requires e0.

        """
        return lambda e0 : e0 /( 1 + math.exp(e0/(GAS_CONSTANT * ROOM_TEMPERATURE)))
    