#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
r"""
Script containing functions for calculating free energy of interaction for 2
SSIPs in solution

The energy for the interaction between 2 SSIPs is:

    2*R * T * ln( [-1 + sqrt(1 + 4K*theta)]/[2K*theta]) - S1 - S2

where R is gas constant,
T is temperature,
K is association constant,
theta is fractional occupancy of the phase,
S1, S2 are the solvation energy of SSIP 1 and 2 respectively.
 
This is just the binding energy contribution, \delta G_{b}.

First term in expression. Derivation:

    Consider SSIPs i and j that interact, K_{ij}!=1, and assume self interaction is zero.

    [i] = [i_{free}] + K[i_{free}][j_{free}] + kvdW[i_{free}]^2

    and noting that [i_{free}] = [j_{free}]
    probability that SSIP is unbound is:

        P_{f} = [i_{free}]/[i] = [-1 + sqrt(1 + 4(K + kvdW)*theta)]/[2(K+ KvdW)*theta]
        
    so the correction per SSIP is:

     R* T * ln( [-1 + sqrt(1 + 4(K+ KvdW)*theta)]/[2(K+ KvdW)*theta])
    Since both SSIPs have a probility of being free then this becomes
    2*R * T * ln( [-1 + sqrt(1 + 4(K+ KvdW)*theta)]/[2(K+ KvdW)*theta])



@author: mark
"""

import logging
import numpy as np
import sys
import solventmapcreator.polynomialanalysis.polynomialvaluecalculator as polyvalcalc
from solventmapcreator.solvationcalculation.interactionenergycalculator import InteractionEnergyCalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

CONVERSION_FROM_J_TO_KJ = 0.001
"""
value for the gas constant, R in JK^-1mol^-1.
https://en.wikipedia.org/wiki/Gas_constant
Multiply by conversion factor to express this in kJK^-1mol^-1, so that
output values are in kJmol^-1.
"""
GAS_CONSTANT = 8.3144598

"""
value of the van der Waals interaction of 2 SSIPs interacting in a
pairwise manner. As detailed in DOI: http://dx.doi.org/10.1039/c3sc22124e.

In kJmol^-1.
"""
EVDW = -5.6

"""
Value of maximum concentration of SSIPs in a phase.
"""
CMAX = 300.0


def calcluate_association_energy_matrix(
    epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
):
    """Iterate through and fills an array with all the solvation energies.

    Parameters
    ----------
    epsilon_i_list : list or np.array
        Values of epsilon_i (x axis).
    epsilon_j_list : list or np.array
        Values of epsilon_j (y asix).
    temperature : float
        In Kelvin.
    theta : float
        Fractional occupancy of the phase
    polynomial_coefficients : np.array
        polynomial coefficents for solvation energy fit.

    Returns
    -------
    association_matrix : np.array.
        2D array of association energy values for each pair of epsilon_i,
        epsilon_j values.

    """
    association_matrix = np.zeros((len(epsilon_i_list), len(epsilon_j_list)))
    for i in range(len(epsilon_i_list)):
        for j in range(len(epsilon_j_list)):
            association_energy = calculate_association_energy(
                epsilon_i_list[i],
                epsilon_j_list[j],
                temperature,
                theta,
                polynomial_coefficients,
            )
            association_matrix[i][j] = association_energy
    return association_matrix


def calculate_association_energy(
    epsilon_i, epsilon_j, temperature, theta, polynomial_coefficients
):
    """This calculates the energy described above
    """
    solvation_energy_i = calculate_solvation_energy(epsilon_i, polynomial_coefficients)
    solvation_energy_j = calculate_solvation_energy(epsilon_j, polynomial_coefficients)

    binding_energy = calculate_binding_energy(epsilon_i, epsilon_j, temperature, theta)
    return binding_energy - solvation_energy_i - solvation_energy_j


def calculate_binding_energy(epsilon_i, epsilon_j, temperature, theta):
    """This evaluates the first term in the association energy calculation.
    This corresponds to the energy of binding between the 2 SSIPs.
    This is in kJ/mol.
    """
    association_constant = calculate_association_constant(
        epsilon_i, epsilon_j, temperature
    )
    # when interactions are very repulsive, then association constant is close to 0.
    if association_constant <= sys.float_info.epsilon:
        association_constant = sys.float_info.epsilon
    fraction_numerator = -1.0 + np.sqrt(1.0 + 4 * association_constant * theta)
    fraction_denominator = 2 * association_constant * theta
    return (
        2
        * (GAS_CONSTANT * CONVERSION_FROM_J_TO_KJ)
        * temperature
        * np.log(fraction_numerator / fraction_denominator)
    )


def calculate_psi_vdw(temperature, theta):
    """Calculate psi_vdw for given temperature and theta.

    Parameters
    ----------
    temperature : float
        temperature in kelvin.
    theta : float
        fractional occupancy of phase.

    Returns
    -------
    psi_vdw : float
        fraction free when only vdw interactions are present.

    """
    K_vdW = calculate_assoc_const(0.0, 0.0, temperature)
    fraction_numerator = -1.0 + np.sqrt(1.0 + 8 *K_vdW * theta)
    fraction_denominator = 4 * K_vdW * theta
    psi_vdw = fraction_numerator / fraction_denominator
    return psi_vdw


def calculate_confinement_energy(temperature, theta):
    r"""This calculates the confinement energy, $\Delta G_c$, for an SSIP. Given by:
       - R* T * ln( [-1 + sqrt(1 + 4K*theta)]/[2K*theta])
    """
    fraction_numerator = -1.0 + np.sqrt(1.0 + 8 * theta)
    fraction_denominator = 4 * theta
    return (
        -(GAS_CONSTANT * CONVERSION_FROM_J_TO_KJ)
        * temperature
        * np.log(fraction_numerator / fraction_denominator)
    )


def calculate_solvation_energy(epsilon, polynomial_coefficients):
    """This calcultes the solvation energy for the given SSIP value, using the
    polynomial coefficients given.
    """
    return polyvalcalc.calculate_polynomial_values(epsilon, polynomial_coefficients)


def calculate_association_constant(epsilon_i, epsilon_j, temperature):
    """This returns the association constant
    """
    k_ij = calculate_assoc_const(epsilon_i, epsilon_j, temperature)
    K_vdW = calculate_assoc_const(0.0, 0.0, temperature)
    total_k = k_ij + K_vdW
    return total_k


def calculate_assoc_const(epsilon_i, epsilon_j, temperature):
    """Calculate the association constant for a single interaction.

    Parameters
    ----------
    epsilon_i : float
        ith SSIP value.
    epsilon_j : float
        jth SSIP value.
    temperature : float
        Temperature in Kelvin.

    Returns
    -------
    float
        Association constant for interaction.

    """
    interaction_energy_298K = epsilon_i * epsilon_j #if epsilon_i* epsilon_j < 0.0 else 0.0
    interaction_energy = None
    if interaction_energy_298K < 0.0:
        interaction_energy = InteractionEnergyCalculator.calculate_interaction_energy_using_spline(interaction_energy_298K, temperature) + EVDW
    else:
        interaction_energy = interaction_energy_298K + EVDW
    exponent = -(interaction_energy * np.power(10, 3)) / (GAS_CONSTANT * temperature)
    return 0.5 * np.exp(exponent)
