#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for reading in the polynomial coefficients, and also the fractional
occupancy, and generating the solvent Maps. Temperature and the ranges of the
two SSIPs forthe x and y axes need to be supplied.

@author: mark
"""

import logging
import copy
import resultsanalysis.resultsoutput.plottinginput as plottinginput
import solventmapcreator.io.polynomialdatareader as polynomialdatareader
import solventmapcreator.io.solvationmapwriter as solvationmapwriter
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator
import solventmapcreator.solvationcalculation.solvationplotinformation as solvationplotinformation

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def create_solvation_diff_plot(
    assoc_filename,
    frac_occ_inf_dict,
    solvent_id,
    polynomial_filename,
    polynomial_order,
    **kwargs
):
    """This creates the solvation plot for the difference.
    """
    input_assoc_data = create_plot_input_assoc_values(assoc_filename, **kwargs)
    epsilon_i_list = input_assoc_data["x_data"][0, :]
    epsilon_j_list = input_assoc_data["y_data"][:, 0]
    input_solv_data = create_plot_input_data_from_files(
        epsilon_i_list,
        epsilon_j_list,
        frac_occ_inf_dict,
        solvent_id,
        polynomial_filename,
        polynomial_order,
        **kwargs
    )
    input_diff_data = copy.deepcopy(input_assoc_data)
    input_diff_data["z_data"] = input_assoc_data["z_data"] - input_solv_data["z_data"]
    input_diff_data["figure_label"] = input_diff_data["figure_label"] + "diff"
    LOGGER.info("input diff data:")
    LOGGER.info(input_diff_data)
    return plot_and_write_contour_map(input_diff_data, **kwargs)


def create_solvation_plot_assoc(filename, **kwargs):
    """This creates solvation plot from association energies.
    """
    input_data = create_plot_input_assoc_values(filename, **kwargs)
    epsilon_i_list = input_data["x_data"][0, :]
    epsilon_j_list = input_data["y_data"][:, 0]
    LOGGER.debug("Input data:")
    LOGGER.debug(input_data)
    matrix_write_out = write_matrix_to_file(
        input_data["figure_label"] + ".csv",
        epsilon_i_list,
        epsilon_j_list,
        input_data["z_data"],
    )
    return plot_and_write_contour_map(input_data, **kwargs), matrix_write_out


def create_solvation_plot(
    epsilon_i_list,
    epsilon_j_list,
    frac_occ_inf_dict,
    solvent_id,
    polynomial_filename,
    polynomial_order,
    **kwargs
):
    """This creates and outputs the solvation plot.
    """
    input_data = create_plot_input_data_from_files(
        epsilon_i_list,
        epsilon_j_list,
        frac_occ_inf_dict,
        solvent_id,
        polynomial_filename,
        polynomial_order,
        **kwargs
    )
    LOGGER.debug("Input data:")
    LOGGER.debug(input_data)
    matrix_write_out = write_matrix_to_file(
        input_data["figure_label"] + ".csv",
        epsilon_i_list,
        epsilon_j_list,
        input_data["z_data"],
    )
    return plot_and_write_contour_map(input_data, **kwargs), matrix_write_out


def plot_and_write_contour_map(input_data, **kwargs):
    """This creates and plots a contour map wit the input given.
    """
    return plottinginput.create_and_write_contour_plot(
        input_data, kwargs.get("fileformat", "svg")
    )


def write_matrix_to_file(filename, x_values, y_values, solvation_matrix):
    """This writes the matrix to file.
    """
    return solvationmapwriter.write_matrix_to_file(
        filename, x_values, y_values, solvation_matrix
    )


def create_plot_input_assoc_values(filename, **kwargs):
    """This creates a plot for the association values.
    """
    x_label = kwargs.get("x_label", r"$\beta$")
    y_label = kwargs.get("y_label", r"$\alpha$")
    solvent_id = kwargs.pop(
        "solvent_id", filename.split("/")[-1].replace("_fgip_associations.xml", "")
    )
    input_data = solvationplotinformation.create_plot_input_data_from_assocs(
        filename, solvent_id, x_label, y_label
    )
    append_output_filename_stem(input_data, solvent_id + "_assoc", **kwargs)
    return input_data


def create_plot_input_data_from_files(
    epsilon_i_list,
    epsilon_j_list,
    frac_occ_inf_dict,
    solvent_id,
    polynomial_filename,
    polynomial_order,
    **kwargs
):
    """This reads in the information and generates the data for plotting.
    """
    temperature = kwargs.get("temperature", 298.0)
    theta = calculate_fractional_occupancy_for_phase(
        frac_occ_inf_dict, solvent_id, temperature
    )
    LOGGER.debug("Solvent ID: %s theta: %.5f",solvent_id, theta)
    polynomial_coefficients = parse_polynomial_data_file_extract_polynomial_coefficients(
        polynomial_filename, polynomial_order
    )
    
    x_label = kwargs.get("x_label", r"$\beta$")
    y_label = kwargs.get("y_label", r"$\alpha$")
    input_data = create_plot_input_data(
        epsilon_i_list,
        epsilon_j_list,
        temperature,
        theta,
        polynomial_coefficients,
        x_label,
        y_label,
    )
    append_output_filename_stem(input_data, solvent_id, **kwargs)
    return input_data


def append_output_filename_stem(plot_input_data, solvent_id, **kwargs):
    """This generates the outputfilename stem for the plot, and appends it to
    the plotting input data.
    """
    plot_input_data["figure_label"] = create_output_filename_stem(
        solvent_id, kwargs.get("directory", "")
    )


def create_output_filename_stem(solvent_id, directory):
    """This generates the outputfilename stem for the plot.
    """
    if directory != "":
        return directory + "/" + solvent_id.replace(",", "_") + "_solv_map"
    else:
        return solvent_id.replace(",", "_") + "_solv_map"


def create_plot_input_data(
    epsilon_i_list,
    epsilon_j_list,
    temperature,
    theta,
    polynomial_coefficients,
    x_label,
    y_label,
):
    """This creates a dictionary containing the input data and labels for plotting.
    """
    return solvationplotinformation.create_plot_input_data(
        epsilon_i_list,
        epsilon_j_list,
        temperature,
        theta,
        polynomial_coefficients,
        x_label,
        y_label,
    )


def calculate_fractional_occupancy_for_phase(
    frac_occ_inf_dict, solvent_id, temperature
):
    """Extract Fractional Occupancy of appropriate phase from dictionary.

    Parameters
    ----------
    frac_occ_inf_dict : dict
        fractional occupancies labelled by solventID and temperature.
    solvent_id : str
        solvent ID.
    temperature : float
        temeprature in Kelvin.

    Returns
    -------
    fractional_occupancy : float
        fractional occupancy.

    """
    return fractionaloccupancycalculator.calculate_fractional_occupancy_for_phase(
        frac_occ_inf_dict, solvent_id, temperature
    )


def parse_polynomial_data_file_extract_polynomial_coefficients(
    filename, polynomial_order
):
    """This parses a polynomial file and gets the coefficients for the given
    polynomial order.
    """
    polynomial_information = parse_polynomial_data_file(filename)[polynomial_order]
    if "positive" in polynomial_information.keys():
        return {
            "positive": polynomial_information["positive"]["coefficients"],
            "negative": polynomial_information["negative"]["coefficients"],
        }
    else:
        return polynomial_information["coefficients"]


def parse_polynomial_data_file(filename):
    """This parses a polynomial data file, and extracts the information
    relating to the polynomial fits, returning a dictionary.
    """
    return polynomialdatareader.parse_polynomial_data_file(filename)
