#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for generating the input data required for plotting the contour map.

@author: mark
"""

import logging
import numpy as np
import solventmapcreator.solvationcalculation.solvationcalculator as solvationcalculator
import phasexmlparser.parsers.associationenergyparser as assocparser

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)


def create_plot_input_data_from_assocs(filename, solvent_id, x_label, y_label):
    """This generates the input from association energy file.
    """
    association_data = assocparser.parse_elements_from_file(filename, solvent_id)
    LOGGER.debug("assoc data: ")
    LOGGER.debug(association_data)
    association_dict = assocparser.restructure_data(association_data)
    LOGGER.debug("restructured data:")
    LOGGER.debug(association_dict)
    input_data_values = assocparser.generate_data_matrices(association_dict)
    LOGGER.info("data values:")
    LOGGER.info(input_data_values)
    input_data_labels = create_input_data_labels(input_data_values, x_label, y_label)
    return {**input_data_values, **input_data_labels}


def create_plot_input_data(
    epsilon_i_list,
    epsilon_j_list,
    temperature,
    theta,
    polynomial_coefficients,
    x_label,
    y_label,
):
    """This creates a dictionary containing the input data and labels for plotting.
    """
    input_data_values = create_input_data_values(
        epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
    )
    input_data_labels = create_input_data_labels(input_data_values, x_label, y_label)
    return {**input_data_values, **input_data_labels}


def create_input_data_labels(input_data_values, x_label, y_label):
    """This creates a dictionary containing the labels with annotations.
    """
    axis_range = get_axis_range(
        input_data_values["x_data"], input_data_values["y_data"]
    )
    line_lists = create_line_lists(axis_range)
    level_min = int(np.floor(input_data_values["z_data"].min()))
    if level_min % 2 == 0:
        level_min = level_min - 1
    levels_list = range(
        level_min, int(np.ceil(input_data_values["z_data"].max())) + 1, 2
    )
    return {
        "x_label": x_label,
        "y_label": y_label,
        "plot_axis_range": axis_range,
        "levels": levels_list,
        **line_lists,
    }


def create_line_lists(axis_range):
    """This creates the lists of values for horizontal and vertical lines.
    """
    x_range = int(np.abs(axis_range[0] - axis_range[1]))
    y_range = int(np.abs(axis_range[2] - axis_range[3]))
    LOGGER.debug("x range: %d", x_range)
    LOGGER.debug("y range: %d", y_range)
    LOGGER.debug("x start: %d", int(np.floor(axis_range[1])))
    LOGGER.debug("y start: %d", int(np.floor(axis_range[2])))
    if y_range <= 5:
        h_lines = [
            int(np.floor(axis_range[2])) + x / 4.0 for x in range(0, 4 * y_range + 1)
        ]
    else:
        h_lines = [
            int(np.floor(axis_range[2])) + x / 2.0 for x in range(0, 2 * y_range + 1)
        ]
    if x_range <= 5:
        v_lines = [
            int(np.floor(axis_range[1])) + x / 4.0 for x in range(0, 4 * x_range + 1)
        ]
    else:
        v_lines = [
            int(np.floor(axis_range[1])) + x / 2.0 for x in range(0, 2 * x_range + 1)
        ]
    LOGGER.debug("h lines: %s", h_lines)
    LOGGER.debug("v lines: %s", v_lines)
    return {"h_lines": h_lines, "v_lines": v_lines}


def create_input_data_values(
    epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
):
    """This calculates the association energy matrix, and also generates the meshgrids ofthe x and y data.
    """
    association_matrix = calculate_association_energy_matrix(
        epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
    )
    x_data, y_data = create_meshgrids(epsilon_i_list, epsilon_j_list)
    return {"x_data": x_data, "y_data": y_data, "z_data": association_matrix}


def get_axis_range(epsilon_i_list, epsilon_j_list):
    """This gets the range of the values in the x and y data.
    """
    return (
        epsilon_i_list.max(),
        epsilon_i_list.min(),
        epsilon_j_list.min(),
        epsilon_j_list.max(),
    )


def create_meshgrids(epsilon_i_list, epsilon_j_list):
    """This creates the mesh grids of the x and y coordinates, to get the right dimensionality for plotting.
    """
    return np.meshgrid(epsilon_i_list, epsilon_j_list, indexing="ij")


def calculate_association_energy_matrix(
    epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
):
    """This creates the matrix of association constants.
    """
    return solvationcalculator.calcluate_association_energy_matrix(
        epsilon_i_list, epsilon_j_list, temperature, theta, polynomial_coefficients
    )
