# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing clustering analysis.

@author: mark
"""

import logging
import numpy as np
import unittest
import solventmapcreator.clusteringanalysis.clustercalculation as clustercalculation

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class ClusterCalculationTestCase(unittest.TestCase):
    """Test case for cluster calculation script.
    """

    def setUp(self):
        """Set up before tests. Using example matrix from the wikipedia page for the UPGMA.
        https://en.wikipedia.org/wiki/UPGMA.
        """
        self.example_distance_matrix = np.array(
            [17, 21, 31, 23, 30, 34, 21, 28, 39, 43]
        )
        self.expected_linkage_matrix_dict = {
            "ward_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 23.459184, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 43.875582, 5.0],
                ]
            ),
            "wpgmc_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 20.31625, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 30.650245, 5.0],
                ]
            ),
            "upgmc_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 20.31625, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 28.321566, 5.0],
                ]
            ),
            "wpgma_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 22.0, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 35.0, 5.0],
                ]
            ),
            "upgma_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 22.0, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 33.0, 5.0],
                ]
            ),
            "max_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [4.0, 5.0, 23.0, 3.0],
                    [2.0, 3.0, 28.0, 2.0],
                    [6.0, 7.0, 43.0, 5.0],
                ]
            ),
            "min_clustering": np.array(
                [
                    [0.0, 1.0, 17.0, 2.0],
                    [2.0, 5.0, 21.0, 3.0],
                    [4.0, 6.0, 21.0, 4.0],
                    [3.0, 7.0, 28.0, 5.0],
                ]
            ),
        }

    def tearDown(self):
        """Clean up after test.
        """
        del self.example_distance_matrix
        del self.expected_linkage_matrix_dict

    def test_clustering_by_all_methods(self):
        """Test to see if expected_dict is produced.
        """
        actual_dict = clustercalculation.clustering_by_all_methods(
            self.example_distance_matrix
        )
        self.assertListEqual(
            sorted(self.expected_linkage_matrix_dict.keys()), sorted(actual_dict.keys())
        )
        for key in self.expected_linkage_matrix_dict.keys():
            np.testing.assert_array_almost_equal(
                self.expected_linkage_matrix_dict[key], actual_dict[key]
            )

    def test_ward_clustering(self):
        """Test to see if expected clustering matrix is produced for
        Ward linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 23.459184, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 43.875582, 5.0],
            ]
        )
        actual_matrix = clustercalculation.ward_clustering(self.example_distance_matrix)
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_wpgmc_clustering(self):
        """Test to see if expected clustering matrix is produced for
        WPGMC linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 20.31625, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 30.650245, 5.0],
            ]
        )
        actual_matrix = clustercalculation.wpgmc_clustering(
            self.example_distance_matrix
        )
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_upgmc_clustering(self):
        """Test to see if expected clustering matrix is produced for
        UPGMC linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 20.31625, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 28.321566, 5.0],
            ]
        )
        actual_matrix = clustercalculation.upgmc_clustering(
            self.example_distance_matrix
        )
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_wpgma_clustering(self):
        """Test to see if expected clustering matrix is produced for
        average/UPGMA linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 22.0, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 35.0, 5.0],
            ]
        )
        actual_matrix = clustercalculation.wpgma_clustering(
            self.example_distance_matrix
        )
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_upgma_clustering(self):
        """Test to see if expected clustering matrix is produced for
        average/UPGMA linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 22.0, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 33.0, 5.0],
            ]
        )
        actual_matrix = clustercalculation.upgma_clustering(
            self.example_distance_matrix
        )
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_max_clustering(self):
        """Test to see if expected clustering matrix is produced for
        maximum linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 23.0, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 43.0, 5.0],
            ]
        )
        actual_matrix = clustercalculation.max_clustering(self.example_distance_matrix)
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_min_clustering(self):
        """Test to see if expected clustering matrix is produced for
        minimum linkages.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [2.0, 5.0, 21.0, 3.0],
                [4.0, 6.0, 21.0, 4.0],
                [3.0, 7.0, 28.0, 5.0],
            ]
        )
        actual_matrix = clustercalculation.min_clustering(self.example_distance_matrix)
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)

    def test_calculate_clustering(self):
        """Test to see if expected clustering matrix is produced.
        """
        expected_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 22.0, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 33.0, 5.0],
            ]
        )
        actual_matrix = clustercalculation.calculate_clustering(
            self.example_distance_matrix, "average"
        )
        np.testing.assert_array_almost_equal(expected_linkage_matrix, actual_matrix)
