# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for running tests on the clusterstatistics script.

@author: mark
"""

import logging
import numpy as np
import unittest
import solventmapcreator.clusteringanalysis.clusterstatistics as clusterstatistics

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class ClusterStatisticsTestCase(unittest.TestCase):
    """Test case for clusterstatistics.
    """

    def setUp(self):
        """Set up for tests.
        """
        self.example_distance_matrix = np.array(
            [17, 21, 31, 23, 30, 34, 21, 28, 39, 43]
        )
        self.example_linkage_matrix = np.array(
            [
                [0.0, 1.0, 17.0, 2.0],
                [4.0, 5.0, 22.0, 3.0],
                [2.0, 3.0, 28.0, 2.0],
                [6.0, 7.0, 33.0, 5.0],
            ]
        )

    def tearDown(self):
        """Clean up after tests.
        """
        del self.example_distance_matrix
        del self.example_linkage_matrix

    def test_calculate_max_inconsistency_stats(self):
        """Test to see if expected max stats are calculated.
        """
        expected_matrix = np.array([17.0, 19.5, 28.0, 28.0])
        inconsistency_matrix = clusterstatistics.calculate_inconsistency(
            self.example_linkage_matrix
        )
        actual_matrix = clusterstatistics.calculate_max_inconsistency_stats(
            self.example_linkage_matrix, inconsistency_matrix, 0
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)

    def test_calculate_max_dist(self):
        """Test to see if expected max dist matrix is produced.
        """
        expected_matrix = np.array([17.0, 22.0, 28.0, 33.0])
        actual_matrix = clusterstatistics.calculate_max_dist(
            self.example_linkage_matrix
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)

    def test_calculate_inconsistency(self):
        """Test to see if expected inconsistency matrix is returned.
        """
        expected_matrix = np.array(
            [
                [17.0, 0.0, 1.0, 0.0],
                [19.5, 3.535534, 2.0, 0.707107],
                [28.0, 0.0, 1.0, 0.0],
                [27.666667, 5.507571, 3.0, 0.968364],
            ]
        )
        actual_matrix = clusterstatistics.calculate_inconsistency(
            self.example_linkage_matrix
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)

    def test_calculate_cophenetic_distance(self):
        """Test to see if cophenetic distance and cophenetic matrix are
        returned as expected.
        """
        expected_distance = 0.73020316123580109
        expected_matrix = np.array(
            [17.0, 33.0, 33.0, 22.0, 33.0, 33.0, 22.0, 28.0, 33.0, 33.0]
        )
        actual_distance, actual_matrix = clusterstatistics.calculate_cophenetic_distance(
            self.example_linkage_matrix, self.example_distance_matrix
        )
        self.assertAlmostEqual(expected_distance, actual_distance)
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
