# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing the matrixinput script.

@author: mark
"""

import logging
import unittest
import numpy as np
import pathlib
import solventmapcreator.clusteringanalysis.matrixinput as matrixinput

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class MatrixInputTestCase(unittest.TestCase):
    """Test case for matrix input script
    """

    def setUp(self):
        """Set up before tests.
        """
        parent_directory = pathlib.Path(__file__).parents[1]
        self.input_filename = (
            (parent_directory / "resources/similarity_matrix_file.csv")
            .absolute()
            .as_posix()
        )
        self.expected_dict = {
            "labels": ["poly_fit", "poly_fit2", "poly_fit3"],
            "value_matrix": np.array(
                [[0.0, 1.0, 4.0], [1.0, 0.0, 3.0], [4.0, 3.0, 0.0]]
            ),
        }

    def tearDown(self):
        """Clean up after tests.
        """
        del self.input_filename
        del self.expected_dict

    def test_parse_similarity_matrix_file_condensed_matrix(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {
            "labels": ["poly_fit", "poly_fit2", "poly_fit3"],
            "value_matrix": np.array(
                [[0.0, 1.0, 4.0], [1.0, 0.0, 3.0], [4.0, 3.0, 0.0]]
            ),
            "condensed_matrix": np.array([1.0, 4.0, 3.0]),
        }
        actual_dict = matrixinput.parse_similarity_matrix_file_condensed_matrix(
            self.input_filename
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        self.assertListEqual(expected_dict["labels"], actual_dict["labels"])
        np.testing.assert_array_almost_equal(
            expected_dict["value_matrix"], actual_dict["value_matrix"]
        )
        np.testing.assert_array_almost_equal(
            expected_dict["condensed_matrix"], actual_dict["condensed_matrix"]
        )

    def test_append_condensed_matrix(self):
        """Test to see if matrix is appended as expected.
        """
        expected_dict = {
            "labels": ["poly_fit", "poly_fit2", "poly_fit3"],
            "value_matrix": np.array(
                [[0.0, 1.0, 4.0], [1.0, 0.0, 3.0], [4.0, 3.0, 0.0]]
            ),
            "condensed_matrix": np.array([1.0, 4.0, 3.0]),
        }
        matrixinput.append_condensed_matrix(self.expected_dict)
        self.assertListEqual(
            sorted(self.expected_dict.keys()), sorted(expected_dict.keys())
        )
        self.assertListEqual(self.expected_dict["labels"], expected_dict["labels"])
        np.testing.assert_array_almost_equal(
            self.expected_dict["value_matrix"], expected_dict["value_matrix"]
        )
        np.testing.assert_array_almost_equal(
            self.expected_dict["condensed_matrix"], expected_dict["condensed_matrix"]
        )

    def test_convert_to_condensed_matrix(self):
        """Test to see if expected matrix is returned.
        """
        expected_matrix = np.array([1.0, 4.0, 3.0])
        actual_matrix = matrixinput.convert_to_condensed_matrix(
            self.expected_dict["value_matrix"]
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)

    def test_parse_similarity_matrix_file(self):
        """Test to see if expected matrix is read in.
        """
        actual_dict = matrixinput.parse_similarity_matrix_file(self.input_filename)
        self.assertListEqual(
            sorted(self.expected_dict.keys()), sorted(actual_dict.keys())
        )
        self.assertListEqual(self.expected_dict["labels"], actual_dict["labels"])
        np.testing.assert_array_almost_equal(
            self.expected_dict["value_matrix"], actual_dict["value_matrix"]
        )
