# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testingthe polynomialdatawriter script.

@author: mark
"""

import logging
import unittest
import numpy as np
import os
import pathlib
from resultsanalysis.dataclasses.datapoint import Datapoint
from resultsanalysis.dataclasses.units import Units
from resultsanalysis.dataclasses.value import Value
from resultsanalysis.dataclasses.valuetype import ValueType
from resultsanalysis.dataclasses.datapoints import Datapoints
import solventmapcreator.io.polynomialdatawriter as polynomialdatawriter

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class PolynomialDataWriterTestCase(unittest.TestCase):
    """Test case for the polynomial data writer.
    """

    def setUp(self):
        """Set up for test cases
        """
        parent_directory = pathlib.Path(__file__).parents[1]
        self.expected_file_name = (
            (parent_directory / "resources/expected_poly_fit.csv").absolute().as_posix()
        )
        self.expected_filename2 = (
            (parent_directory / "resources/expected_poly_fit2.csv")
            .absolute()
            .as_posix()
        )
        self.datapoints = Datapoints()
        self.datapoints.addDatapointList(
            [
                Datapoint.from_values(
                    Value(1.0, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    Value(1.0, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    "EXAMPLE1",
                ),
                Datapoint.from_values(
                    Value(2.0, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    Value(2.1, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    "EXAMPLE2",
                ),
                Datapoint.from_values(
                    Value(0.5, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    Value(0.5, "Log(Sw)", ValueType.undefined, Units.dimensionless),
                    "EXAMPLE3",
                ),
            ]
        )
        self.expected_polynomial_data = {
            1: {
                "coefficients": np.array([-0.05, 1.071428571429]),
                "RMSE": 0.015430334996209221,
                "order": 1,
                "covar": np.array([0.0007142857142857]),
            }
        }

    def tearDown(self):
        """Tear down after tests
        """
        del self.datapoints
        del self.expected_polynomial_data
        del self.expected_file_name
        if os.path.isfile("actual_poly_fit.csv"):
            os.remove("actual_poly_fit.csv")
        if os.path.isfile("actual_poly_fit2.csv"):
            os.remove("actual_poly_fit2.csv")

    def test_write_poly_data_to_file_split_fit(self):
        """Test to see if expected file is written out.
        """
        actual_file_name = "actual_poly_fit2.csv"
        poly_file_out = polynomialdatawriter.write_poly_data_to_file_split_fit(
            self.datapoints,
            [1],
            actual_file_name,
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
        )
        self.assertEqual(0, poly_file_out)
        with open(self.expected_filename2, "r") as exp_file:
            exp_file_lines = exp_file.read()
            with open(actual_file_name, "r") as act_file:
                act_file_lines = act_file.read()
                self.assertMultiLineEqual(act_file_lines, exp_file_lines)

    def test_write_poly_data_to_file(self):
        """Test to see if expected file is produced.
        """
        actual_file_name = "actual_poly_fit.csv"
        poly_file_out = polynomialdatawriter.write_poly_data_to_file(
            self.datapoints, [1], actual_file_name
        )
        self.assertEqual(0, poly_file_out)
        with open(self.expected_file_name, "r") as exp_file:
            exp_file_lines = exp_file.read()
            with open(actual_file_name, "r") as act_file:
                act_file_lines = act_file.read()
                self.assertMultiLineEqual(act_file_lines, exp_file_lines)

    def test_write_poly_fit_data_to_file(self):
        """Test to see if expected file is produced.
        """
        actual_file_name = "actual_poly_fit.csv"
        poly_file_out = polynomialdatawriter.write_poly_fit_data_to_file(
            self.expected_polynomial_data, actual_file_name
        )
        self.assertEqual(0, poly_file_out)
        with open(self.expected_file_name, "r") as exp_file:
            exp_file_lines = exp_file.read()
            with open(actual_file_name, "r") as act_file:
                act_file_lines = act_file.read()
                self.assertMultiLineEqual(act_file_lines, exp_file_lines)

    def test_generate_poly_fit_data_order_list_split_fit(self):
        """Test to see if expected dictionary is returned.
        """
        expected_poly_dict = {
            1: {
                "positive": self.expected_polynomial_data[1],
                "negative": self.expected_polynomial_data[1],
            }
        }
        actual_poly_dict = polynomialdatawriter.generate_poly_fit_data_order_list_split_fit(
            self.datapoints,
            [1],
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
        )
        self.assertListEqual(
            sorted(expected_poly_dict.keys()), sorted(actual_poly_dict.keys())
        )
        for poly_order in expected_poly_dict.keys():
            expected_mix_dictionary = expected_poly_dict[poly_order]
            actual_mix_dictionary = actual_poly_dict[poly_order]
            self.assertListEqual(
                sorted(expected_mix_dictionary.keys()),
                sorted(actual_mix_dictionary.keys()),
            )
            for key in expected_mix_dictionary.keys():
                actual_dictionary = actual_mix_dictionary[key]
                expected_dictionary = expected_mix_dictionary[key]
                self.assertListEqual(
                    sorted(expected_dictionary.keys()), sorted(actual_dictionary.keys())
                )
                for key in expected_dictionary.keys():
                    if key == "RMSE":
                        self.assertAlmostEqual(
                            expected_dictionary[key], actual_dictionary[key]
                        )
                    elif key == "order":
                        self.assertEqual(
                            expected_dictionary[key], actual_dictionary[key]
                        )
                    else:
                        np.testing.assert_array_almost_equal(
                            expected_dictionary[key], actual_dictionary[key]
                        )

    def test_generate_poly_fit_data_order_list(self):
        """Test to see if expected dictionary is produced.
        """
        actual_poly_dictionary = polynomialdatawriter.generate_poly_fit_data_order_list(
            self.datapoints, [1]
        )
        self.assertListEqual(
            sorted(self.expected_polynomial_data.keys()),
            sorted(actual_poly_dictionary.keys()),
        )
        for poly_order in self.expected_polynomial_data.keys():
            expected_dictionary = self.expected_polynomial_data[poly_order]
            actual_dictionary = actual_poly_dictionary[poly_order]
            for key in expected_dictionary.keys():
                if key == "RMSE":
                    self.assertAlmostEqual(
                        expected_dictionary[key], actual_dictionary[key]
                    )
                elif key == "order":
                    self.assertEqual(expected_dictionary[key], actual_dictionary[key])
                else:
                    np.testing.assert_array_almost_equal(
                        expected_dictionary[key], actual_dictionary[key]
                    )

    def test_generate_polynomial_fit_data_positive_negative_separate_fit(self):
        """Test to see if expected dict is returned.
        """
        expected_mix_dictionary = {
            "positive": self.expected_polynomial_data[1],
            "negative": self.expected_polynomial_data[1],
        }
        actual_mix_dictionary = polynomialdatawriter.generate_polynomial_fit_data_positive_negative_separate_fit(
            self.datapoints,
            1,
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
            ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"],
        )
        self.assertListEqual(
            sorted(expected_mix_dictionary.keys()), sorted(actual_mix_dictionary.keys())
        )
        for key in expected_mix_dictionary.keys():
            actual_dictionary = actual_mix_dictionary[key]
            expected_dictionary = expected_mix_dictionary[key]
            self.assertListEqual(
                sorted(expected_dictionary.keys()), sorted(actual_dictionary.keys())
            )
            for key in expected_dictionary.keys():
                if key == "RMSE":
                    self.assertAlmostEqual(
                        expected_dictionary[key], actual_dictionary[key]
                    )
                elif key == "order":
                    self.assertEqual(expected_dictionary[key], actual_dictionary[key])
                else:
                    np.testing.assert_array_almost_equal(
                        expected_dictionary[key], actual_dictionary[key]
                    )

    def test_generate_polynomial_fit_data_subset(self):
        """Test to see if subset is fitted to and expected dict is returned.
        """
        expected_dictionary = self.expected_polynomial_data[1]
        actual_dictionary = polynomialdatawriter.generate_polynomial_fit_data_subset(
            self.datapoints, 1, ["EXAMPLE1", "EXAMPLE2", "EXAMPLE3"]
        )
        self.assertListEqual(
            sorted(expected_dictionary.keys()), sorted(actual_dictionary.keys())
        )
        for key in expected_dictionary.keys():
            if key == "RMSE":
                self.assertAlmostEqual(expected_dictionary[key], actual_dictionary[key])
            elif key == "order":
                self.assertEqual(expected_dictionary[key], actual_dictionary[key])
            else:
                np.testing.assert_array_almost_equal(
                    expected_dictionary[key], actual_dictionary[key]
                )

    def test_generate_polynomial_fit_data(self):
        """Test to see if expected dictionary is produced.
        """
        actual_dictionary = polynomialdatawriter.generate_polynomial_fit_data(
            self.datapoints, 1
        )
        expected_dictionary = self.expected_polynomial_data[1]
        self.assertListEqual(
            sorted(expected_dictionary.keys()), sorted(actual_dictionary.keys())
        )
        for key in expected_dictionary.keys():
            if key == "RMSE":
                self.assertAlmostEqual(expected_dictionary[key], actual_dictionary[key])
            elif key == "order":
                self.assertEqual(expected_dictionary[key], actual_dictionary[key])
            else:
                np.testing.assert_array_almost_equal(
                    expected_dictionary[key], actual_dictionary[key]
                )
