# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing the polynomial compactor.

@author: mark
"""

import logging
import unittest
import os
import pathlib
import numpy as np
import solventmapcreator.io.polynomialfilecompactor as polycompact

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class PolynomialFileCompactorTestCase(unittest.TestCase):
    """Test case for polynomial compaction.
    """

    def setUp(self):
        """Set up for tests.
        """
        self.maxDiff = None
        parent_directory = pathlib.Path(__file__).parents[1]
        self.filelist = [
            (parent_directory / "resources/water_poly_fit_split.csv")
            .absolute()
            .as_posix()
        ]
        self.actual_data = polycompact.read_polyfiles(
            self.filelist, suffix="_poly_fit_split.csv"
        )
        self.actual_csv = "actualcompactfile.csv"
        self.actual_tex = "actualcompactfile.tex"
        self.expected_csv = (
            (parent_directory / "resources/expectedcompactfile.csv")
            .absolute()
            .as_posix()
        )
        self.expected_tex = (
            (parent_directory / "resources/expectedcompactfile.tex")
            .absolute()
            .as_posix()
        )

    def tearDown(self):
        """Clean up after tests.
        """
        if os.path.isfile(self.actual_csv):
            os.remove(self.actual_csv)
        if os.path.isfile(self.actual_tex):
            os.remove(self.actual_tex)

    def test_compact_polynomial_files(self):
        """Test to see if expected
        """
        polycompact.compact_polynomial_files(
            self.filelist, 4, "actualcompactfile", suffix="_poly_fit_split.csv"
        )
        with open(self.actual_tex, "r") as actualfile:
            actual_file_contents = actualfile.read()
            with open(self.expected_tex, "r") as expfile:
                expected_file_contents = expfile.read()
                self.assertMultiLineEqual(expected_file_contents, actual_file_contents)

    def test_convert_to_tex_table(self):
        """Test to see if conversion was done as expected.
        """
        data = polycompact.create_header_line(4) + polycompact.create_sorted_list(
            self.actual_data, 4
        )
        polycompact.write_info_to_csv_file("actualcompactfile.csv", data)
        polycompact.convert_to_tex_table(self.actual_csv, self.actual_tex)
        with open(self.actual_tex, "r") as actualfile:
            actual_file_contents = actualfile.read()
            with open(self.expected_tex, "r") as expfile:
                expected_file_contents = expfile.read()
                self.assertMultiLineEqual(expected_file_contents, actual_file_contents)

    def test_write_info_to_csv_file(self):
        """Test to see if file is written as expected.
        """
        data = polycompact.create_header_line(4) + polycompact.create_sorted_list(
            self.actual_data, 4
        )
        polycompact.write_info_to_csv_file("actualcompactfile.csv", data)
        with open(self.actual_csv, "r") as actualfile:
            actual_file_contents = actualfile.read()
            with open(self.expected_csv, "r") as expfile:
                expected_file_contents = expfile.read()
                self.assertMultiLineEqual(expected_file_contents, actual_file_contents)

    def test_create_header_line(self):
        """Test to see if header line created as expected.
        """
        expected_list = [["Molecule Name", "Region", "0th", "1th", "2th", "3th", "4th"]]
        actual_list = polycompact.create_header_line(4)
        self.assertListEqual(expected_list, actual_list)

    def test_create_sorted_list(self):
        """Test to see if sorted list is produced.
        """
        expected_list = [
            ["water", "positive", -0.89928914951439998, -0.3969324545381],
            [
                "water",
                "negative",
                6.3245322833520001,
                2.8927088358900002,
                -0.001756642146001,
            ],
        ]
        actual_list = polycompact.create_sorted_list(self.actual_data, 2)
        self.assertListEqual(expected_list, actual_list)

    def test_read_polyfiles(self):
        """Test to see if files read as expected.
        """
        expected_poly_dict = {
            2: {
                "positive": {
                    "order": 2,
                    "RMSE": 0.0,
                    "covar": 1.207745983893,
                    "coefficients": np.array([-0.89928915, -0.39693245]),
                },
                "negative": {
                    "order": 2,
                    "RMSE": 0.0188856511,
                    "covar": 0.001783339090217,
                    "coefficients": np.array(
                        [6.32453228e00, 2.89270884e00, -1.75664215e-03]
                    ),
                },
            },
            4: {
                "positive": {
                    "order": 4,
                    "RMSE": 0.0,
                    "covar": 1.351574765483,
                    "coefficients": np.array(
                        [-1.3179762, -0.12678612, -0.01515573, -0.00203797]
                    ),
                },
                "negative": {
                    "order": 4,
                    "RMSE": 0.0,
                    "covar": 5.038842703124,
                    "coefficients": np.array(
                        [
                            2.29062758e00,
                            -9.85582950e-02,
                            -6.48066028e-03,
                            -1.54567335e-04,
                        ]
                    ),
                },
            },
        }
        actual_poly_dictionary = self.actual_data["water"]
        for poly_order in expected_poly_dict.keys():
            expected_dictionary = expected_poly_dict[poly_order]
            actual_dictionary = actual_poly_dictionary[poly_order]

            for region, exp_dict in expected_dictionary.items():
                for key in exp_dict.keys():
                    if key == "RMSE" or key == "covar":
                        self.assertAlmostEqual(
                            exp_dict[key], actual_dictionary[region][key]
                        )
                    elif key == "order":
                        self.assertEqual(exp_dict[key], actual_dictionary[region][key])
                    else:
                        np.testing.assert_array_almost_equal(
                            exp_dict[key], actual_dictionary[region][key]
                        )
