# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script containing tests for the solvationenergyreader module.

@author: mark
"""

import logging
import unittest
import pathlib
import solventmapcreator.io.solvationenergyreader as solvationenergyreader
from resultsanalysis.dataclasses.datapoints import Datapoints
from resultsanalysis.dataclasses.datapoint import Datapoint
from resultsanalysis.dataclasses.units import Units
from resultsanalysis.dataclasses.valuetype import ValueType
from resultsanalysis.dataclasses.value import Value

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class SolvationEnergyReaderTestCase(unittest.TestCase):
    """Test Case for the solvationenergyreader module.
    """

    def setUp(self):
        """Set up for tests
        """
        parent_directory = pathlib.Path(__file__).parents[1]
        self.free_energy_file = (
            (parent_directory / "resources/energyvaluestest2.xml").absolute().as_posix()
        )
        self.binding_energy_file = (
            (parent_directory / "resources/energyvaluestest.xml").absolute().as_posix()
        )
        self.solvation_binding_info_dict = solvationenergyreader.parse_binding_energy_info(
            self.binding_energy_file
        )
        self.solvation_free_info_dict = solvationenergyreader.parse_free_energy_info(
            self.free_energy_file
        )
        self.expected_datapoints = Datapoints()
        self.expected_datapoints.addDatapoint(
            Datapoint.from_values(
                Value(
                    -14.100, r"\epsilon_{i}", ValueType.undefined, Units.dimensionless
                ),
                Value(
                    -34.8195308084233,
                    r"\Delta G_{b,i}",
                    ValueType.undefined,
                    Units.kj_per_mol,
                ),
                "-14.100solute",
            )
        )
        self.expected_free_energy_datapoints = Datapoints()
        self.expected_free_energy_datapoints.addDatapoint(
            Datapoint.from_values(
                Value(
                    -14.100, r"\epsilon_{i}", ValueType.undefined, Units.dimensionless
                ),
                Value(
                    -34.8195308084233,
                    r"\Delta G_{S,i}",
                    ValueType.undefined,
                    Units.kj_per_mol,
                ),
                "-14.100solute",
            )
        )

    def tearDown(self):
        """Tear down after tests.
        """
        del self.solvation_binding_info_dict
        del self.solvation_free_info_dict
        del self.expected_datapoints
        del self.expected_free_energy_datapoints

    def test_parse_free_energy_from_file(self):
        """Test to see if expected Datapoints object is created from reading of
        a file.
        """
        actual_datapoints = solvationenergyreader.parse_free_energy_from_file(
            self.free_energy_file
        )
        self.assertEqual(self.expected_free_energy_datapoints, actual_datapoints)

    def test_parse_binding_energy_from_file(self):
        """Test to see if expected Datapoints object is created from reading of
        a file.
        """
        actual_datapoints = solvationenergyreader.parse_binding_energy_from_file(
            self.binding_energy_file
        )
        self.assertEqual(self.expected_datapoints, actual_datapoints)

    def test_create_free_energy_datapoints_with_values(self):
        """Test to see if expected Datapoints object is created.
        """
        actual_datapoints = solvationenergyreader.create_free_energy_datapoints_with_values(
            self.solvation_free_info_dict
        )
        self.assertEqual(self.expected_free_energy_datapoints, actual_datapoints)

    def test_create_binding_datapoints_with_values(self):
        """Test to see if expected Datapoints object is created.
        """
        actual_datapoints = solvationenergyreader.create_binding_datapoints_with_values(
            self.solvation_binding_info_dict
        )
        self.assertEqual(self.expected_datapoints, actual_datapoints)

    def test_parse_free_energy_info(self):
        """Test to see if expected free energy values are extracted from file.
        """
        expected_dict = {
            "total_energy": -34.8195308084233,
            "value_type": "MOLEFRACTION",
            "to_solvent_id": "water",
            "from_solvent_id": "",
            "molecule_id": "-14.100solute",
        }
        actual_dict_dict = self.solvation_free_info_dict
        self.assertListEqual(["-14.100solute//water"], sorted(actual_dict_dict.keys()))
        actual_dict = actual_dict_dict["-14.100solute//water"]
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key, value in expected_dict.items():
            if key == "total_energy":
                self.assertAlmostEqual(value, actual_dict[key])
            else:
                self.assertEqual(value, actual_dict[key])

    def test_parse_binding_energy_info(self):
        """Test to see if expected free energy values are extracted from file.
        """
        expected_dict = {
            "total_energy": -34.8195308084233,
            "value_type": "MOLEFRACTION",
            "to_solvent_id": "water",
            "from_solvent_id": "",
            "molecule_id": "-14.100solute",
        }
        actual_dict_dict = self.solvation_binding_info_dict
        self.assertListEqual(["-14.100solute//water"], sorted(actual_dict_dict.keys()))
        actual_dict = actual_dict_dict["-14.100solute//water"]
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key, value in expected_dict.items():
            if key == "total_energy":
                self.assertAlmostEqual(value, actual_dict[key])
            else:
                self.assertEqual(value, actual_dict[key])
