# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing multidimensionclustering methods.

@author: mark
"""

import logging
import numpy as np
import unittest
import solventmapcreator.polynomialanalysis.multidimensionclustering as multicluster

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class MultiDimensionClusteringTestCase(unittest.TestCase):
    """Test case for multidimension clustering methods.
    """

    def setUp(self):
        """Test set up.
        """
        self.poly_data_by_solvent_id = {
            "poly_fit": {
                2: {
                    "coefficients": np.array([0.0, 0.0, 1.0]),
                    "RMSE": 0.0254303,
                    "order": 2,
                    "covar": 0.0008143,
                }
            },
            "poly_fit2": {
                2: {
                    "coefficients": np.array([1.0, 0.0, 1.0]),
                    "RMSE": 0.0354303,
                    "order": 2,
                    "covar": 0.0008143,
                }
            },
            "poly_fit3": {
                2: {
                    "coefficients": np.array([0.0, 0.0, 2.0]),
                    "RMSE": 0.0354303,
                    "order": 2,
                    "covar": 0.0008143,
                }
            },
        }
        self.x_values = np.array([-2.0, -1.0, 0.0, 1.0, 2.0])
        self.poly_values_dict = {
            "poly_fit": np.array([4.0, 1.0, 0.0, 1.0, 4.0]),
            "poly_fit2": np.array([5.0, 2.0, 1.0, 2.0, 5.0]),
            "poly_fit3": np.array([8.0, 2.0, 0.0, 2.0, 8.0]),
        }

    def tearDown(self):
        """Clean up after tests.
        """
        del self.poly_data_by_solvent_id
        del self.x_values
        del self.poly_values_dict

    def test_condense_matrix_to_mean(self):
        """Test to see if matrix is condensed along correct axis.
        """
        expected_matrix = np.array(
            [[0.0, 0.691741, 0.5], [0.691741, 0.0, 0.873773], [0.5, 0.873773, 0.0]]
        )
        actual_matrix = multicluster.condense_matrix_to_mean(
            np.array(
                [
                    [[0.0, 0.0], [0.383482, 1.0], [1.0, 0.0]],
                    [[0.383482, 1.0], [0.0, 0.0], [0.747545, 1.0]],
                    [[1.0, 0.0], [0.747545, 1.0], [0.0, 0.0]],
                ]
            )
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)

    def test_generate_normalised_rmsd_matrix_stack(self):
        """Test to see if expected matrix is generated.
        """
        expected_matrix = np.array(
            [
                [[0.0, 0.0], [0.383482, 1.0], [1.0, 0.0]],
                [[0.383482, 1.0], [0.0, 0.0], [0.747545, 1.0]],
                [[1.0, 0.0], [0.747545, 1.0], [0.0, 0.0]],
            ]
        )
        actual_matrix, norm_factors = multicluster.generate_normalised_rmsd_matrix_stack(
            [self.x_values, [0.0]], 2, self.poly_data_by_solvent_id
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
        np.testing.assert_array_almost_equal(np.array([2.607681, 1.0]), norm_factors)

    def test_generate_normalised_rmsd_matrix(self):
        """Test to see if expected matrix is generated.
        """
        expected_matrix = np.array(
            [[0.0, 0.383482, 1.0], [0.383482, 0.0, 0.747545], [1.0, 0.747545, 0.0]]
        )
        actual_matrix, norm_factors = multicluster.generate_normalised_rmsd_matrix(
            self.x_values, 2, self.poly_data_by_solvent_id
        )
        np.testing.assert_array_almost_equal(expected_matrix, actual_matrix)
        np.testing.assert_array_almost_equal(np.array([2.607681]), norm_factors)
