# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Created on Thu Jul  6 21:13:47 2017

@author: mark
"""

import logging
import unittest
import numpy as np
import solventmapcreator.polynomialanalysis.polynomialdataanalysis as polynomialdataanalysis

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class PolynomialDataAnalysisTestCase(unittest.TestCase):
    """Test case for the polynomialdataanalysis script
    """

    def setUp(self):
        """set up for class.
        """
        self.expected_data_by_solvent_id = {
            "expected_poly_fit": {
                1: {
                    "coefficients": np.array([-0.05, 1.07142857]),
                    "RMSE": 0.0154303,
                    "order": 1,
                    "covar": 0.0007143,
                },
                2: {
                    "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                    "RMSE": 0.0254303,
                    "order": 1,
                    "covar": 0.0008143,
                },
            },
            "expected_poly_fit2": {
                1: {
                    "coefficients": np.array([-0.05, 1.07142857]),
                    "RMSE": 0.0254303,
                    "order": 1,
                    "covar": 0.007143,
                },
                2: {
                    "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                    "RMSE": 0.0354303,
                    "order": 1,
                    "covar": 0.0008143,
                },
            },
        }
        self.exp_data_by_solv_id_pos_neg = {
            "expected_poly_fit": {
                1: {
                    "positive": {
                        "coefficients": np.array([-0.05, 1.07142857]),
                        "RMSE": 0.0154303,
                        "order": 1,
                        "covar": 0.0007143,
                    },
                    "negative": {
                        "coefficients": np.array([-0.05, 1.07142857]),
                        "RMSE": 0.0154303,
                        "order": 1,
                        "covar": 0.0007143,
                    },
                },
                2: {
                    "positive": {
                        "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                        "RMSE": 0.0254303,
                        "order": 1,
                        "covar": 0.0008143,
                    },
                    "negative": {
                        "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                        "RMSE": 0.0254303,
                        "order": 1,
                        "covar": 0.0008143,
                    },
                },
            },
            "expected_poly_fit2": {
                1: {
                    "positive": {
                        "coefficients": np.array([-0.05, 1.07142857]),
                        "RMSE": 0.0254303,
                        "order": 1,
                        "covar": 0.007143,
                    },
                    "negative": {
                        "coefficients": np.array([-0.05, 1.07142857]),
                        "RMSE": 0.0254303,
                        "order": 1,
                        "covar": 0.007143,
                    },
                },
                2: {
                    "positive": {
                        "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                        "RMSE": 0.0354303,
                        "order": 1,
                        "covar": 0.0008143,
                    },
                    "negative": {
                        "coefficients": np.array([-0.05, 1.07142857, 0.1]),
                        "RMSE": 0.0354303,
                        "order": 1,
                        "covar": 0.0008143,
                    },
                },
            },
        }
        self.rmse_dict = polynomialdataanalysis.get_best_poly_fit_rmse(
            self.expected_data_by_solvent_id
        )
        self.covar_dict = polynomialdataanalysis.get_best_poly_fit_covar(
            self.expected_data_by_solvent_id
        )

    def tearDown(self):
        """tear down for tests.
        """
        del self.expected_data_by_solvent_id
        del self.rmse_dict
        del self.covar_dict

    def test_get_pos_neg_rmse_by_order_asarrays(self):
        """Test to see if expected dict of arrays is returned.
        """
        expected_dict = {
            "positive": np.array([[1.0, 2.0], [0.020430299999999998, 0.0304303]]),
            "negative": np.array([[1.0, 2.0], [0.020430299999999998, 0.0304303]]),
        }
        actual_dict = polynomialdataanalysis.get_pos_neg_rmse_by_order_asarrays(
            self.exp_data_by_solv_id_pos_neg, [1, 2]
        )
        for key in expected_dict.keys():
            np.testing.assert_array_almost_equal(expected_dict[key], actual_dict[key])

    def test_get_mean_rmse_by_order_asarray(self):
        """Test to see if expected array is returned.
        """
        expected_array = np.array([[1.0, 2.0], [0.020430299999999998, 0.0304303]])
        actual_array = polynomialdataanalysis.get_mean_rmse_by_order_asarray(
            self.expected_data_by_solvent_id, [1, 2]
        )
        np.testing.assert_array_almost_equal(expected_array, actual_array)

    def test_get_pos_neg_rmse_by_order(self):
        """
        """
        expected_pos_neg_dict = {
            "positive": {1: 0.020430299999999998, 2: 0.0304303},
            "negative": {1: 0.020430299999999998, 2: 0.0304303},
        }
        actual_pos_neg_dict = polynomialdataanalysis.get_pos_neg_rmse_by_order(
            self.exp_data_by_solv_id_pos_neg, [1, 2]
        )
        self.assertListEqual(sorted(expected_pos_neg_dict), sorted(actual_pos_neg_dict))
        for pn_key in expected_pos_neg_dict.keys():
            expected_dict = expected_pos_neg_dict[pn_key]
            actual_dict = actual_pos_neg_dict[pn_key]
            for key in expected_dict.keys():
                self.assertAlmostEqual(expected_dict[key], actual_dict[key])

    def test_get_mean_rmse_by_order(self):
        """Test to see if expected dict is returned.
        """
        expected_dict = {1: 0.020430299999999998, 2: 0.0304303}
        actual_dict = polynomialdataanalysis.get_mean_rmse_by_order(
            self.expected_data_by_solvent_id, [1, 2]
        )
        self.assertEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            self.assertAlmostEqual(expected_dict[key], actual_dict[key])

    def test_get_mean_negative_rmse(self):
        """Test to see if expected mean is returned
        """
        expected_mean = 0.020430299999999998
        actual_mean = polynomialdataanalysis.get_mean_negative_rmse(
            self.exp_data_by_solv_id_pos_neg, 1
        )
        self.assertAlmostEqual(expected_mean, actual_mean)

    def test_get_mean_positive_rmse(self):
        """Test to see if expected mean is returned
        """
        expected_mean = 0.020430299999999998
        actual_mean = polynomialdataanalysis.get_mean_positive_rmse(
            self.exp_data_by_solv_id_pos_neg, 1
        )
        self.assertAlmostEqual(expected_mean, actual_mean)

    def test_get_mean_rmse(self):
        """Test to see if expected mean is returned
        """
        expected_mean = 0.020430299999999998
        actual_mean = polynomialdataanalysis.get_mean_rmse(
            self.expected_data_by_solvent_id, 1
        )
        self.assertAlmostEqual(expected_mean, actual_mean)

    def test_get_total_sorted_by_alt_metric(self):
        """Test to see if sorted correctly
        """
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.rmse_dict
        )
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.covar_dict
        )
        ordered_id_dict = polynomialdataanalysis.generate_lists_by_alt_metric(
            [1, 2], rmse_solvent_id_dict, covar_solvent_id_dict
        )
        expected_tuple = ((1, 2), ((1, 0), (1, 0)))
        actual_tuple = polynomialdataanalysis.get_total_sorted_by_alt_metric(
            [1, 2], ordered_id_dict
        )
        self.assertEqual(expected_tuple, actual_tuple)

    def test_get_total_by_fit_order(self):
        """Test to see if expected totals are returned.
        """
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.rmse_dict
        )
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.covar_dict
        )
        expected_tuple_rmse = ((1,), (2,))
        actual_tuple_rmse = polynomialdataanalysis.get_total_by_fit_order(
            rmse_solvent_id_dict
        )
        self.assertEqual(expected_tuple_rmse, actual_tuple_rmse)
        expected_tuple_covar = ((1, 2), (1, 1))
        actual_tuple_covar = polynomialdataanalysis.get_total_by_fit_order(
            covar_solvent_id_dict
        )
        self.assertEqual(expected_tuple_covar, actual_tuple_covar)

    def test_generate_lists_by_alt_metric(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {1: {1: ["expected_poly_fit"], 2: ["expected_poly_fit2"]}}
        rmse_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.rmse_dict
        )
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.covar_dict
        )
        actual_dict = polynomialdataanalysis.generate_lists_by_alt_metric(
            [1, 2], rmse_solvent_id_dict, covar_solvent_id_dict
        )
        self.assertDictEqual(expected_dict, actual_dict)
        expected_dict2 = {1: {1: ["expected_poly_fit"]}, 2: {1: ["expected_poly_fit2"]}}
        actual_dict2 = polynomialdataanalysis.generate_lists_by_alt_metric(
            [1, 2], covar_solvent_id_dict, rmse_solvent_id_dict
        )
        self.assertDictEqual(expected_dict2, actual_dict2)

    def test_extract_lists_by_order_of_alt_metric(self):
        """Test to if expected dict is produced.
        """
        expected_dict = {1: ["expected_poly_fit"], 2: ["expected_poly_fit2"]}
        rmse_list = ["expected_poly_fit", "expected_poly_fit2"]
        covar_solvent_id_dict = polynomialdataanalysis.get_solvent_id_by_order(
            self.covar_dict
        )
        actual_dict = polynomialdataanalysis.extract_lists_by_order_of_alt_metric(
            [1, 2], rmse_list, covar_solvent_id_dict
        )
        self.assertDictEqual(expected_dict, actual_dict)

    def test_get_solvent_id_by_order(self):
        """Test to see if expected dictionary is returned
        """
        expected_dict = {1: ["expected_poly_fit", "expected_poly_fit2"]}
        actual_dict = polynomialdataanalysis.get_solvent_id_by_order(self.rmse_dict)
        self.assertDictEqual(expected_dict, actual_dict)

    def test_get_solvent_id(self):
        """Test to see if expected list is returned.
        """
        expected_list = ["expected_poly_fit", "expected_poly_fit2"]
        actual_list = polynomialdataanalysis.get_solvent_id(self.rmse_dict[1])
        self.assertListEqual(expected_list, actual_list)

    def test_get_smallest_order_below_threshold(self):
        """Test to see if expected dict is returned.
        """
        expected_dict = {
            1: [("expected_poly_fit", 0.0154303), ("expected_poly_fit2", 0.0254303)]
        }
        actual_dict = polynomialdataanalysis.get_smallest_order_below_threshold(
            self.expected_data_by_solvent_id, 0.03
        )
        self.assertDictEqual(expected_dict, actual_dict)

    def test_extract_smallest_order_below_threshold(self):
        """Test to see if expected values are returned.
        """
        expected_order = 1
        expected_rmse = 0.0154303
        actual_order, actual_rmse = polynomialdataanalysis.extract_smallest_order_below_threshold(
            self.expected_data_by_solvent_id["expected_poly_fit"], 0.016
        )
        self.assertAlmostEqual(expected_rmse, actual_rmse)
        self.assertEqual(expected_order, actual_order)

    def test_get_best_poly_fit_rmse(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {
            1: [("expected_poly_fit", 0.0154303), ("expected_poly_fit2", 0.0254303)]
        }
        actual_dict = polynomialdataanalysis.get_best_poly_fit_rmse(
            self.expected_data_by_solvent_id
        )
        self.assertDictEqual(expected_dict, actual_dict)

    def test_extract_best_poly_fit_rmse(self):
        """Test to see if expected value is extracted.
        """
        expected_order = 1
        expected_rmse = 0.0154303
        actual_order, actual_rmse = polynomialdataanalysis.extract_best_poly_fit_rmse(
            self.expected_data_by_solvent_id["expected_poly_fit"]
        )
        self.assertAlmostEqual(expected_rmse, actual_rmse)
        self.assertEqual(expected_order, actual_order)

    def test_get_best_poly_fit_covar(self):
        """Test to see if expected dict is produced.
        """
        expected_dict = {
            1: [("expected_poly_fit", 0.0007143)],
            2: [("expected_poly_fit2", 0.0008143)],
        }
        actual_dict = polynomialdataanalysis.get_best_poly_fit_covar(
            self.expected_data_by_solvent_id
        )
        self.assertDictEqual(expected_dict, actual_dict)

    def test_extract_best_poly_fit_covar(self):
        """Test to see if expected value is extracted.
        """
        expected_order = 1
        expected_covar = 0.0007143
        actual_order, actual_covar = polynomialdataanalysis.extract_best_poly_fit_covar(
            self.expected_data_by_solvent_id["expected_poly_fit"]
        )
        self.assertAlmostEqual(expected_covar, actual_covar)
        self.assertEqual(expected_order, actual_order)
