# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing the polynomialplotting script.

@author: mark
"""

import logging
import os
import numpy as np
import unittest
import csv
import pandas
import pathlib
import solventmapcreator.io.solvationenergyreader as solvationenergyreader
import solventmapcreator.polynomialanalysis.polynomialplotting as polynomialplotting

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class PolynomialPlottingTestCase(unittest.TestCase):
    """Test case for polynomialplotting script
    """

    def setUp(self):
        """Set up for tests
        """
        self.maxDiff = None
        self.parent_directory = pathlib.Path(__file__).parents[1]
        self.poly_fit_file = (
            (self.parent_directory / "resources/water_poly_fit.csv")
            .absolute()
            .as_posix()
        )
        self.poly_fit_split_file = (
            (self.parent_directory / "resources/water_poly_fit_split.csv")
            .absolute()
            .as_posix()
        )
        self.poly_fit_subset_file = (
            (self.parent_directory / "resources/water_poly_fit_subset.csv")
            .absolute()
            .as_posix()
        )
        self.free_energy_file = (
            (self.parent_directory / "resources/water_freeenergy.xml")
            .absolute()
            .as_posix()
        )
        self.binding_energy_file = (
            (self.parent_directory / "resources/water.xml").absolute().as_posix()
        )
        self.expected_datapoints = solvationenergyreader.parse_binding_energy_from_file(
            self.binding_energy_file
        )
        self.expected_free_energydatapoints = solvationenergyreader.parse_free_energy_from_file(
            self.free_energy_file
        )

    def tearDown(self):
        """Clean up after tests.
        """
        del self.expected_datapoints
        if os.path.isfile("actual_water_poly_fit.csv"):
            os.remove("actual_water_poly_fit.csv")
        if os.path.isfile("actual_water_poly_fit_subset.csv"):
            os.remove("actual_water_poly_fit_subset.csv")
        if os.path.isfile("actual_water_poly_fit_split.csv"):
            os.remove("actual_water_poly_fit_split.csv")

    def test_parse_free_energies_create_plot_input_data(self):
        """Test to see expected plot data is returned.
        """
        expected_dict = {
            "figure_label": (self.parent_directory / "resources/water_freeenergy_fe")
            .absolute()
            .as_posix(),
            "plot_axis_range": [-15.4, 7.2, -38.635444, 1.918583],
            "polynomial coefficients": np.array(
                [
                    5.18106356e-01,
                    2.54615444e-01,
                    -3.76651666e-01,
                    -2.06376692e-02,
                    -3.76010096e-04,
                ]
            ),
            "polynomial order": 4,
            "x_axis_range": [-15.4, 7.2000000000000002],
            "x_data": np.array(
                [-14.1, -5.4, -4.3, 7.2, 0.5, -0.1, -2.4, 1.2, -9.1, -11.1, -15.4]
            ),
            "x_label": r"$\epsilon_{i}$",
            "y_axis_range": [-38.635444, 0.918583],
            "y_data": np.array(
                [
                    -34.819531,
                    -9.315465,
                    -6.170784,
                    -25.844114,
                    0.658868,
                    0.918583,
                    -1.40826,
                    -0.442984,
                    -20.143788,
                    -26.01378,
                    -38.635444,
                ]
            ),
            "y_label": r"$\Delta G_{S,i}/kJmol^{-1}$",
        }
        actual_dict = polynomialplotting.parse_free_energies_create_plot_input_data(
            self.free_energy_file, 4
        )
        LOGGER.debug(actual_dict["x_data"].tolist())
        LOGGER.debug(actual_dict["y_data"].tolist())
        self.assertListEqual(sorted(actual_dict.keys()), sorted(expected_dict.keys()))
        for key in actual_dict.keys():
            LOGGER.debug("Key: %s", key)
            if key == "y_data" or key == "x_data" or key == "polynomial coefficients":
                LOGGER.debug("assert equal array x and y data")
                np.testing.assert_array_almost_equal(
                    actual_dict[key], expected_dict[key]
                )
            elif (
                key == "plot_axis_range"
                or key == "x_axis_range"
                or key == "y_axis_range"
            ):
                np.testing.assert_array_almost_equal(
                    np.array(actual_dict[key]), np.array(expected_dict[key])
                )
            else:
                LOGGER.debug("assert equal string")
                self.assertEqual(actual_dict[key], expected_dict[key])
        # test to see if new functionality is supported
        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        actual_dict2 = polynomialplotting.parse_free_energies_create_plot_input_data(
            self.free_energy_file, 4, pos_set_list, neg_set_list
        )
        self.assertListEqual(sorted(actual_dict2.keys()), sorted(expected_dict.keys()))
        for key in actual_dict.keys():
            LOGGER.debug("Key: %s", key)
            if key == "y_data" or key == "x_data":
                LOGGER.debug("assert equal array x and y data")
                np.testing.assert_array_almost_equal(
                    actual_dict2[key], expected_dict[key]
                )
            elif key == "polynomial coefficients":
                self.assertListEqual(
                    ["negative", "positive"],
                    sorted(actual_dict2["polynomial coefficients"].keys()),
                )
                negative_coeffs = np.array(
                    [
                        5.038843e00,
                        2.290628e00,
                        -9.855830e-02,
                        -6.480660e-03,
                        -1.545673e-04,
                    ]
                )
                positive_coeffs = np.array(
                    [1.351575, -1.317976, -0.126786, -0.015156, -0.002038]
                )
                np.testing.assert_array_almost_equal(
                    negative_coeffs, actual_dict2["polynomial coefficients"]["negative"]
                )
                np.testing.assert_array_almost_equal(
                    positive_coeffs, actual_dict2["polynomial coefficients"]["positive"]
                )
            elif (
                key == "plot_axis_range"
                or key == "x_axis_range"
                or key == "y_axis_range"
            ):
                np.testing.assert_array_almost_equal(
                    np.array(actual_dict2[key]), np.array(expected_dict[key])
                )
            else:
                LOGGER.debug("assert equal string")
                self.assertEqual(expected_dict[key], actual_dict2[key])

    def test_parse_binding_energies_create_plot_input_data(self):
        """Test to see expected plot data is returned.
        """
        expected_dict = {
            "figure_label": (self.parent_directory / "resources/water_be")
            .absolute()
            .as_posix(),
            "plot_axis_range": [-15.4, 7.2, -38.635444, 1.918583],
            "polynomial coefficients": np.array(
                [
                    5.18106356e-01,
                    2.54615444e-01,
                    -3.76651666e-01,
                    -2.06376692e-02,
                    -3.76010096e-04,
                ]
            ),
            "polynomial order": 4,
            "x_axis_range": [-15.4, 7.2000000000000002],
            "x_data": np.array(
                [-14.1, -5.4, -4.3, 7.2, 0.5, -0.1, -2.4, 1.2, -9.1, -11.1, -15.4]
            ),
            "x_label": r"$\epsilon_{i}$",
            "y_axis_range": [-38.635444, 0.918583],
            "y_data": np.array(
                [
                    -34.819531,
                    -9.315465,
                    -6.170784,
                    -25.844114,
                    0.658868,
                    0.918583,
                    -1.40826,
                    -0.442984,
                    -20.143788,
                    -26.01378,
                    -38.635444,
                ]
            ),
            "y_label": r"$\Delta G_{b,i}/kJmol^{-1}$",
        }
        actual_dict = polynomialplotting.parse_binding_energies_create_plot_input_data(
            self.binding_energy_file, 4
        )
        LOGGER.debug(actual_dict["x_data"].tolist())
        LOGGER.debug(actual_dict["y_data"].tolist())
        self.assertListEqual(sorted(actual_dict.keys()), sorted(expected_dict.keys()))
        for key in actual_dict.keys():
            LOGGER.debug("Key: %s", key)
            if key == "y_data" or key == "x_data" or key == "polynomial coefficients":
                LOGGER.debug("assert equal array x and y data")
                np.testing.assert_array_almost_equal(
                    actual_dict[key], expected_dict[key]
                )
            elif (
                key == "plot_axis_range"
                or key == "x_axis_range"
                or key == "y_axis_range"
            ):
                np.testing.assert_array_almost_equal(
                    np.array(actual_dict[key]), np.array(expected_dict[key])
                )
            else:
                LOGGER.debug("assert equal string")
                self.assertEqual(actual_dict[key], expected_dict[key])
        # test to see if new functionality is supported
        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        actual_dict2 = polynomialplotting.parse_binding_energies_create_plot_input_data(
            self.binding_energy_file, 4, pos_set_list, neg_set_list
        )
        self.assertListEqual(sorted(actual_dict2.keys()), sorted(expected_dict.keys()))
        for key in actual_dict.keys():
            LOGGER.debug("Key: %s", key)
            if key == "y_data" or key == "x_data":
                LOGGER.debug("assert equal array x and y data")
                np.testing.assert_array_almost_equal(
                    actual_dict2[key], expected_dict[key]
                )
            elif key == "polynomial coefficients":
                self.assertListEqual(
                    ["negative", "positive"],
                    sorted(actual_dict2["polynomial coefficients"].keys()),
                )
                negative_coeffs = np.array(
                    [
                        5.038843e00,
                        2.290628e00,
                        -9.855830e-02,
                        -6.480660e-03,
                        -1.545673e-04,
                    ]
                )
                positive_coeffs = np.array(
                    [1.351575, -1.317976, -0.126786, -0.015156, -0.002038]
                )
                np.testing.assert_array_almost_equal(
                    negative_coeffs, actual_dict2["polynomial coefficients"]["negative"]
                )
                np.testing.assert_array_almost_equal(
                    positive_coeffs, actual_dict2["polynomial coefficients"]["positive"]
                )
            elif (
                key == "plot_axis_range"
                or key == "x_axis_range"
                or key == "y_axis_range"
            ):
                np.testing.assert_array_almost_equal(
                    np.array(actual_dict2[key]), np.array(expected_dict[key])
                )
            else:
                LOGGER.debug("assert equal string")
                self.assertEqual(actual_dict2[key], expected_dict[key])

    def test_generate_poly_pos_neg_separate_fit(self):
        """Test to see if expected dict is returned.
        """

        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        negative_coeffs = np.array(
            [5.038843e00, 2.290628e00, -9.855830e-02, -6.480660e-03, -1.545673e-04]
        )
        positive_coeffs = np.array(
            [1.351575, -1.317976, -0.126786, -0.015156, -0.002038]
        )
        actual_dict = polynomialplotting.generate_poly_pos_neg_separate_fit(
            self.expected_datapoints, 4, pos_set_list, neg_set_list
        )
        LOGGER.debug("Actual Dict")
        LOGGER.debug(actual_dict)
        self.assertListEqual(["negative", "positive"], sorted(actual_dict.keys()))
        np.testing.assert_array_almost_equal(negative_coeffs, actual_dict["negative"])
        np.testing.assert_array_almost_equal(positive_coeffs, actual_dict["positive"])

    def test_free_energy_parse_poly_data_to_file_split_fit(self):
        """Test to see if expected fit is done on the positive and negative subsets
        after the file was read in.
        """
        actual_file_name = "actual_water_poly_fit_split.csv"
        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        poly_file_out = polynomialplotting.parse_free_energy_poly_data_to_file_split_fit(
            self.free_energy_file, [2, 4], actual_file_name, pos_set_list, neg_set_list
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_split_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_binding_energy_parse_poly_data_to_file_split_fit(self):
        """Test to see if expected fit is done on the positive and negative subsets
        after the file was read in.
        """
        actual_file_name = "actual_water_poly_fit_split.csv"
        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        poly_file_out = polynomialplotting.parse_binding_energy_poly_data_to_file_split_fit(
            self.binding_energy_file,
            [2, 4],
            actual_file_name,
            pos_set_list,
            neg_set_list,
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_split_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_write_poly_data_to_file_split_fit(self):
        """Test to see if expected_fit is done on the positive and negative subsets.
        """
        actual_file_name = "actual_water_poly_fit_split.csv"
        neg_set_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        pos_set_list = ["0.500solute", "1.200solute", "7.200solute"]
        poly_file_out = polynomialplotting.write_poly_data_to_file_split_fit(
            self.expected_datapoints,
            [2, 4],
            actual_file_name,
            pos_set_list,
            neg_set_list,
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_split_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_parse_free_energies_and_output_poly_data_subset(self):
        """Test to see if expected_fit is done on a smaller subset of points.
        """
        actual_file_name = "actual_water_poly_fit_subset.csv"
        subset_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        poly_file_out = polynomialplotting.parse_free_energies_and_output_poly_data_subset(
            self.free_energy_file, [2, 4], actual_file_name, subset_list
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_subset_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_parse_binding_energies_and_output_poly_data_subset(self):
        """Test to see if expected_fit is done on a smaller subset of points.
        """
        actual_file_name = "actual_water_poly_fit_subset.csv"
        subset_list = [
            "-5.400solute",
            "-4.300solute",
            "-9.100solute",
            "-11.100solute",
            "-15.400solute",
        ]
        poly_file_out = polynomialplotting.parse_binding_energies_and_output_poly_data_subset(
            self.binding_energy_file, [2, 4], actual_file_name, subset_list
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_subset_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_parse_free_energies_and_output_poly_data(self):
        """Test to see if expected polynomial fit information is outputted to file.
        """
        actual_file_name = "actual_water_poly_fit.csv"
        poly_file_out = polynomialplotting.parse_free_energies_and_output_poly_data(
            self.free_energy_file, [2, 4], actual_file_name
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_parse_binding_energies_and_output_poly_data(self):
        """Test to see if expected polynomial fit information is outputted to file.
        """
        actual_file_name = "actual_water_poly_fit.csv"
        poly_file_out = polynomialplotting.parse_binding_energies_and_output_poly_data(
            self.binding_energy_file, [2, 4], actual_file_name
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )
        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_write_poly_data_to_file(self):
        """Test to see if expected polynomial fit data is written to file.
        """
        actual_file_name = "actual_water_poly_fit.csv"
        poly_file_out = polynomialplotting.write_poly_data_to_file(
            self.expected_datapoints, [2, 4], actual_file_name
        )
        self.assertEqual(0, poly_file_out)
        exp_file = pandas.read_csv(
            self.poly_fit_file,
            sep="\t",
            quoting=csv.QUOTE_NONE,
            names=list(range(0, 9)),
        )
        act_file = pandas.read_csv(
            actual_file_name, sep="\t", quoting=csv.QUOTE_NONE, names=list(range(0, 9))
        )

        self.assertTrue(exp_file.columns.equals(act_file.columns))
        self.assertTrue(exp_file.loc[0].equals(act_file.loc[0]))
        for i in range(1, exp_file.shape[0]):
            self.assertTrue(exp_file.loc[i, :1].equals(act_file.loc[i, :1]))
            np.testing.assert_almost_equal(
                exp_file.loc[i, 2:].values.astype(float),
                act_file.loc[i, 2:].values.astype(float),
            )

    def test_parse_free_energy_from_file_with_data_arrays(self):
        """Test to see if expected datapoints are read in.
        """
        actual_datapoints = polynomialplotting.parse_free_energy_from_file_with_data_arrays(
            self.free_energy_file
        )
        self.assertEqual(self.expected_free_energydatapoints, actual_datapoints)

    def test_parse_binding_energy_from_file_with_data_arrays(self):
        """Test to see if expected datapoints are read in.
        """
        actual_datapoints = polynomialplotting.parse_binding_energy_from_file_with_data_arrays(
            self.binding_energy_file
        )
        self.assertEqual(self.expected_datapoints, actual_datapoints)
