# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing gamma calculator

@author: mark
"""


import logging
import unittest
import pathlib
import numpy as np
import solventmapcreator.solvationcalculation.gammacalculator as gammacalculator
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class GammaCalculatorTestCase(unittest.TestCase):
    """Test case for methods."""

    def setUp(self):
        """Set up before tests."""
        parent_directory = pathlib.Path(__file__).parents[1]
        self.poly_file = (
            (parent_directory / "resources/waterbinding_poly_fit_split.csv")
            .absolute()
            .as_posix()
        )
        self.assoc_file = (
            (parent_directory / "resources/associationvaluesgamma.xml")
            .absolute()
            .as_posix()
        )
        self.directory = (parent_directory / "resources").absolute().as_posix()
        self.frac_occ_file = (parent_directory / "resources/fractional_occupancy.xml").absolute().as_posix()
        self.frac_occ_inf_dict = fractionaloccupancycalculator.read_fractional_occupancy_information(self.frac_occ_file)
        self.assocdata = gammacalculator.parse_elements_from_file(self.assoc_file)
        self.gamma_info = {
            "epsilon_i": 1.3,
            "epsilon_j": 1.3,
            "temperature": 298.0,
            "solute_concentration": 0.001,
        }

    def tearDown(self):
        """Clean up after tests."""
        del self.frac_occ_inf_dict
        del self.assocdata
        del self.gamma_info

    def test_calculate_mean_and_sigma(self):
        """Test to see if information calculated matches expectation."""
        expected_mean = 9.6728178895399122
        expected_sigma = 0.0
        actual_mean, actual_sigma = gammacalculator.calculate_mean_and_sigma(
            {"water": 9.6728178895399122}
        )
        self.assertAlmostEqual(expected_mean, actual_mean)
        self.assertAlmostEqual(expected_sigma, actual_sigma)

    def test_calculate_simulation_gamma_for_solvent_list(self):
        """Test to see if expected information is returned."""
        expected_dict = {"water": 12.405685407733245}
        actual_dict = gammacalculator.calculate_simulation_gamma_for_solvent_list(
            self.gamma_info,
            ["water"],
            self.frac_occ_inf_dict,
            self.assocdata,
            self.directory,
            4,
        )
        for key, value in actual_dict.items():
            self.assertAlmostEqual(expected_dict[key], value)

    def test_calculate_simulation_gamma(self):
        """Test to see if expected value is returned."""
        expected_value = 12.405685407733245

        actual_value = gammacalculator.calculate_simulation_gamma(
            self.gamma_info,
            "water",
            self.frac_occ_inf_dict,
            self.assocdata,
            self.directory,
            4,
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_association_energy(self):
        """Test to seeif expected value is returned."""
        expected_value = -9.0219865959507608
        actual_value = gammacalculator.calculate_association_energy(
            1.0, 1.0, 298.0, 1.0, np.array([0.5, 1.0])
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_analytical_gamma_for_solvent_list(self):
        """Test to see if expected dict is produced."""
        expected_dict = {"water": 11.662151076166985}
        actual_dict = gammacalculator.calculate_analytical_gamma_for_solvent_list(
            1.3, 1.3, 298.0, self.frac_occ_inf_dict, ["water"]
        )
        for key, value in actual_dict.items():
            self.assertAlmostEqual(expected_dict[key], value)

    def test_calculate_analytical_gamma_for_solvent(self):
        """Test to see if expected value is returned."""
        expected_value = 11.662151076166985
        actual_value = gammacalculator.calculate_analytical_gamma_for_solvent(
            1.3, 1.3, 298.0, self.frac_occ_inf_dict, "water"
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_analytical_gamma(self):
        """Test to see if expected value returned."""
        expected_value = 11.662151076166985
        actual_value = gammacalculator.calculate_analytical_gamma(
            1.3, 1.3, 0.7380033333333333, 298.0
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_parse_elements_from_file(self):
        """Test to see if elements extracted as expected."""
        expected_elements = {
            "water": {
                "solute1": "1.300solute1",
                "solute2": "1.300solute2",
                "solvent_id": "water",
                "temp": "298.000KELVIN",
                "value": 10.222313287855103,
            }
        }
        actual_elements = self.assocdata
        self.assertDictEqual(expected_elements, actual_elements)

    def test_create_poly_filename_from_solvent_id(self):
        """Test to see if expected file created."""
        expected_name = "waterbinding_poly_fit_split.csv"
        actual_name = gammacalculator.create_poly_filename_from_solvent_id(
            None, "water"
        )
        self.assertEqual(expected_name, actual_name)

    def test_parse_polynomial_data_file_extract_polynomial_coefficients(self):
        """Test to see if expected coefficients extracted."""
        expected_data = {
            "negative": np.array(
                [2.29062758e00, -9.85582950e-02, -6.48066028e-03, -1.54567335e-04]
            ),
            "positive": np.array([-1.3179762, -0.12678612, -0.01515573, -0.00203797]),
        }
        actual_data = gammacalculator.parse_polynomial_data_file_extract_polynomial_coefficients(
            self.poly_file, 4
        )
        for key, entry in actual_data.items():
            with self.subTest(key=key):
                np.testing.assert_almost_equal(expected_data[key], entry)

    def test_calculate_fractional_occupancy_for_phase(self):
        """Test to see if ffractional occupancy returned."""
        expected_theta = 0.7380033333333333
        actual_theta = gammacalculator.calculate_fractional_occupancy_for_phase(
            self.frac_occ_inf_dict, "water", 298.0
        )
        self.assertAlmostEqual(expected_theta, actual_theta)
