# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""Test case for interaction energy calculator class.

@author: mark
"""

import logging
import unittest
from solventmapcreator.solvationcalculation.interactionenergycalculator import InteractionEnergyCalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

class InteractionEnergyCalculatorTestCase(unittest.TestCase):
    """Test case for InteractionEnergyCalculator."""

    def test_calculate_interaction_energy(self):
        """Test of calculate interaction energy method.

        Returns
        -------
        None.

        """
        parameters = [[-1.0, 298.0, -0.5995519650366715],
		[-10.0, 298.0, -9.826385042109182],
		[-50.0, 298.0, -49.99999991391315],
		[-175.0, 298.0, -175.0],
		[-395.0, 298.0, -395.0],
		[-151.0, 298.0, -151.0],
		[-72.0, 298.0, -71.99999999998273],
		[-60.5, 298.0, -60.49999999849591],
		[-50.8, 298.0, -50.79999993667051],
		[-50.8, 500.0, -50.79974943294584],
		[-60.5, 500.0, -60.49997106090202]]
        for param_set in parameters:
            with self.subTest(params=param_set):
                actual_value = InteractionEnergyCalculator.calculate_interaction_energy(param_set[0], param_set[1])
                self.assertAlmostEqual(param_set[2], actual_value)
    def test_calculate_interaction_energy_rt(self):
        """Test to calculate interaction energy at room temperature.

        Returns
        -------
        None.

        """
        parameters = [[-1.0, -0.5995519650366715],
		[-10.0, -9.826385042109182],
		[-50.0, -49.99999991391315],
		[-175.0, -175.0],
		[-395.0, -395.0],
		[-151.0, -151.0],
		[-72.0, -71.99999999998273],
		[-60.5, -60.49999999849591],
		[-50.8, -50.79999993667051]]
        for param_set in parameters:
            with self.subTest(params=param_set):
                actual_value = InteractionEnergyCalculator.calculate_interaction_energy_rt(param_set[0])
                self.assertAlmostEqual(param_set[1], actual_value)
    def test_calculate_interaction_energy_using_spline(self):
        """Test calculate interaction energy using spline.

        Returns
        -------
        None.

        """
        parameters = [[-0.5001008995, 298.0, -0.5001008999048229],
		[-10.0, 298.0, -10.0],
		[-50.0, 298.0, -50.0],
		[-90.58876365, 298.0, -90.58876365],
		[-213.2096117643254, 298.0, -213.2096117643254],
		[-77.799898, 298.0, -77.799898],
		[-36.52302627, 298.0, -36.52302627],
		[-30.61929909, 298.0, -30.61929909],
		[-25.6603762, 298.0, -25.6603762],
		[-30.61929909, 500.0, -30.60006666053905],
		[-25.6603762, 500.0, -25.607782197269387]]
        for param_set in parameters:
            with self.subTest(params=param_set):
                actual_value = InteractionEnergyCalculator.calculate_interaction_energy_using_spline(param_set[0], param_set[1])
                self.assertAlmostEqual(param_set[2], actual_value,5)
    def test_calculate_interaction_energy_az(self):
        """Test calculate interaction energy at absolute zero.

        Returns
        -------
        None.

        """
        parameters = [[-0.5001008995, -0.85434846],
		[-5.010089938, -5.544628921197275],
		[-25.25224022, -25.25318623150945],
		[-90.58876365, -90.58876365000002],
		[-213.2096117643254, -213.2096117643254],
		[-77.799898, -77.7998980000018],
		[-36.52302627, -36.52304074979894],
		[-30.61929909, -30.61943060708467],
		[-25.6603762, -25.66119153559956]]
        for param_set in parameters:
            with self.subTest(params=param_set):
                actual_value = InteractionEnergyCalculator.calculate_interaction_energy_az(param_set[0])
                LOGGER.debug("type actual value: %s", type(actual_value))
                self.assertAlmostEqual(param_set[1], actual_value,5)