# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing the functions in the solvationmapgenerator script
that assemble the data. The functions that write plots are not tested.

@author: mark
"""

import logging
import unittest
import os
import pathlib
import numpy as np
import solventmapcreator.solvationcalculation.solvationmapgenerator as solvationmapgenerator
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class SolvationMapGeneratorTestCase(unittest.TestCase):
    """Test case for solvationmapgenerator tests.
    """

    def setUp(self):
        """Set up before tests.
        """
        self.expected_data_by_solvent_id = {
            "expected_poly_fit": {
                1: {
                    "coefficients": np.array([-0.05, 1.07142857]),
                    "RMSE": 0.015430334996209221,
                    "order": 1,
                    "covar": 0.0007143,
                }
            }
        }
        self.epsilon_i_list = np.array([float(x) for x in range(3)])
        self.epsilon_j_list = np.array([float(x) for x in range(2)])
        parent_directory = pathlib.Path(__file__).parents[1]
        self.assoc_filename = (
            (parent_directory / "resources/associationvalues.xml").absolute().as_posix()
        )
        self.similarity_mat_file = (
            (parent_directory / "resources/similmatrix.csv").absolute().as_posix()
        )
        self.poly_fit = (
            (parent_directory / "resources/expected_poly_fit.csv").absolute().as_posix()
        )
        self.frac_occ_file = (parent_directory / "resources/fractional_occupancy.xml").absolute().as_posix()
        self.frac_occ_inf_dict = fractionaloccupancycalculator.read_fractional_occupancy_information(self.frac_occ_file)

    def tearDown(self):
        """clean up after tests.
        """
        del self.expected_data_by_solvent_id
        del self.epsilon_i_list
        del self.epsilon_j_list
        del self.assoc_filename
        if os.path.isfile("actualsimilmatrix.csv"):
            os.remove("actualsimilmatrix.csv")

    def test_write_matrix_to_file(self):
        """Test to see if matrix written to file.
        """
        actual_file_name = "actualsimilmatrix.csv"
        simil_out_file = solvationmapgenerator.write_matrix_to_file(
            actual_file_name,
            self.epsilon_i_list,
            self.epsilon_j_list,
            np.array(
                [[-4.332289, -5.403718], [-5.403718, -5.757629], [-6.475146, -6.167992]]
            ),
        )
        self.assertEqual(0, simil_out_file)
        with open(self.similarity_mat_file, "r") as exp_file:
            exp_file_lines = exp_file.readlines()
            with open(actual_file_name, "r") as act_file:
                act_file_lines = act_file.readlines()
                self.assertListEqual(act_file_lines, exp_file_lines)

    def test_create_plot_input_assoc_values(self):
        """Test to see if expected information is generated.
        """
        expected_dict = {
            "figure_label": "water_assoc_solv_map",
            "x_data": np.array([[1.3, 1.3]]),
            "y_data": np.array([[1.3, 4.9]]),
            "z_data": np.array([[10.222313, 14.902313]]),
            "x_label": "$\\beta$",
            "y_label": "$\\alpha$",
            "plot_axis_range": (1.3, 1.3, 1.3, 4.9),
            "levels": [9, 11, 13, 15],
            "h_lines": [
                1.0,
                1.25,
                1.5,
                1.75,
                2.0,
                2.25,
                2.5,
                2.75,
                3.0,
                3.25,
                3.5,
                3.75,
                4.0,
            ],
            "v_lines": [1.0],
        }
        actual_dict = solvationmapgenerator.create_plot_input_assoc_values(
            self.assoc_filename, solvent_id="water"
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            with self.subTest(key=key):
                if key == "plot_axis_range":
                    np.testing.assert_array_almost_equal(
                        np.array(expected_dict[key]), np.array(actual_dict[key])
                    )
                elif "levels" == key or "_lines" in key:
                    np.testing.assert_array_equal(
                        np.array(expected_dict[key]), np.array(actual_dict[key])
                    )
                elif "_data" in key:
                    np.testing.assert_array_almost_equal(
                        expected_dict[key], actual_dict[key]
                    )
                else:
                    self.assertEqual(expected_dict[key], actual_dict[key])

    def test_create_plot_input_data_from_files(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {
            "x_data": np.array([[0, 0], [1, 1], [2, 2]]),
            "y_data": np.array([[0, 1], [0, 1], [0, 1]]),
            "z_data": np.array(
                [[-5.673465, -6.744894], [-6.744894, -7.453301], [-7.816322, -8.244948]]
            ),
            "x_label": "$\\beta$",
            "y_label": "$\\alpha$",
            "plot_axis_range": (2.0, 0.0, 0.0, 1.0),
            "figure_label": "water_solv_map",
            "levels": [-9, -7, -5],
            "h_lines": [0.0, 0.25, 0.5, 0.75, 1.0],
            "v_lines": [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0],
        }
        actual_dict = solvationmapgenerator.create_plot_input_data_from_files(
            self.epsilon_i_list,
            self.epsilon_j_list,
            self.frac_occ_inf_dict,
            "water",
            self.poly_fit,
            1,
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            if key == "plot_axis_range":
                np.testing.assert_array_almost_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            elif "_data" in key or key == "levels":
                np.testing.assert_array_almost_equal(
                    expected_dict[key], actual_dict[key]
                )
            else:
                self.assertEqual(expected_dict[key], actual_dict[key])

    def test_append_output_filename_stem(self):
        """Test to see if filename stem is appended.
        """
        expected_dict = {"figure_label": "water_solv_map"}
        actual_dict = {}
        solvationmapgenerator.append_output_filename_stem(actual_dict, "water")
        self.assertDictEqual(expected_dict, actual_dict)

    def test_create_output_filename_stem(self):
        """Test to see if expected file name stem is produced.
        """
        expected_filename_stem = "water__solv_map"
        actual_filename_stem = solvationmapgenerator.create_output_filename_stem(
            "water,", ""
        )
        self.assertEqual(expected_filename_stem, actual_filename_stem)
        expected_filename_stem2 = "dir/water__solv_map"
        actual_filename_stem2 = solvationmapgenerator.create_output_filename_stem(
            "water,", "dir"
        )
        self.assertEqual(expected_filename_stem2, actual_filename_stem2)

    def test_create_plot_input_data(self):
        """Test to see if expected input data is returned.
        """
        expected_dict = {
            "x_data": np.array([[0, 0], [1, 1], [2, 2]]),
            "y_data": np.array([[0, 1], [0, 1], [0, 1]]),
            "z_data": np.array(
                [[-7.396882, -8.396882], [-8.396882, -9.021987], [-9.396882, -9.732173]]
            ),
            "x_label": "x_label",
            "y_label": "y_label",
            "plot_axis_range": (2.0, 0.0, 0.0, 1.0),
            "levels": [-11, -9, -7],
            "h_lines": [0.0, 0.25, 0.5, 0.75, 1.0],
            "v_lines": [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0],
        }
        actual_dict = solvationmapgenerator.create_plot_input_data(
            self.epsilon_i_list,
            self.epsilon_j_list,
            298.0,
            1.0,
            np.array([0.5, 1.0]),
            "x_label",
            "y_label",
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            if key == "plot_axis_range":
                np.testing.assert_array_almost_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            elif "_data" in key or key == "levels":
                np.testing.assert_array_almost_equal(
                    expected_dict[key], actual_dict[key]
                )
            else:
                self.assertEqual(expected_dict[key], actual_dict[key])

    def test_calculate_fractional_occupancy_for_phase(self):
        """Test to see if expected fractional occupancy is returned.
        """
        expected_value = (55.35 * 4.0 / 300.0) + (0.001 / 300)
        actual_value = solvationmapgenerator.calculate_fractional_occupancy_for_phase(
            self.frac_occ_inf_dict, "water", 298.0
            )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_parse_polynomial_data_file_extract_polynomial_coefficients(self):
        """Test to see if expected array is returned.
        """
        expected_array = self.expected_data_by_solvent_id["expected_poly_fit"][1][
            "coefficients"
        ]
        actual_array = solvationmapgenerator.parse_polynomial_data_file_extract_polynomial_coefficients(
            self.poly_fit, 1
        )
        np.testing.assert_array_almost_equal(expected_array, actual_array)

    def test_parse_polynomial_data_file(self):
        """Test to see if expected dictionary is reutrned.
        """
        expected_poly_dictionary = self.expected_data_by_solvent_id["expected_poly_fit"]
        actual_poly_dictionary = solvationmapgenerator.parse_polynomial_data_file(
            self.poly_fit
        )
        self.assertListEqual(
            sorted(expected_poly_dictionary.keys()),
            sorted(actual_poly_dictionary.keys()),
        )
        for poly_order in expected_poly_dictionary.keys():
            expected_dictionary = expected_poly_dictionary[poly_order]
            actual_dictionary = actual_poly_dictionary[poly_order]
            self.assertListEqual(
                sorted(expected_dictionary.keys()), sorted(actual_dictionary.keys())
            )
            for key in expected_dictionary.keys():
                if key == "RMSE" or key == "covar":
                    self.assertAlmostEqual(
                        expected_dictionary[key], actual_dictionary[key]
                    )
                elif key == "order":
                    self.assertEqual(expected_dictionary[key], actual_dictionary[key])
                else:
                    np.testing.assert_array_almost_equal(
                        expected_dictionary[key], actual_dictionary[key]
                    )
