# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script containing tests for the solvationplotinformation script.

@author: mark
"""

import logging
import unittest
import numpy as np
import pathlib
import solventmapcreator.solvationcalculation.solvationplotinformation as solvationplotinformation

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class SolvationPlotInformationTestCase(unittest.TestCase):
    """Test Case for the solvationplotinformation script.
    """

    def setUp(self):
        """Set up for tests.
        """
        parent_directory = pathlib.Path(__file__).parents[1]
        self.epsilon_i_list = np.array([float(x) for x in range(3)])
        self.epsilon_j_list = np.array([float(x) for x in range(2)])
        self.assoc_filename = (
            (parent_directory / "resources/associationvalues.xml").absolute().as_posix()
        )

    def tearDown(self):
        """clean up after tests.
        """
        del self.epsilon_i_list
        del self.epsilon_j_list
        del self.assoc_filename

    def test_create_plot_input_data_from_assocs(self):
        """Test to see if expected information is created.
        """
        expected_dict = {
            "x_data": np.array([[1.3, 1.3]]),
            "y_data": np.array([[1.3, 4.9]]),
            "z_data": np.array([[10.222313, 14.902313]]),
            "x_label": "x_label",
            "y_label": "y_label",
            "plot_axis_range": (1.3, 1.3, 1.3, 4.9),
            "levels": [9, 11, 13, 15],
            "h_lines": [
                1.0,
                1.25,
                1.5,
                1.75,
                2.0,
                2.25,
                2.5,
                2.75,
                3.0,
                3.25,
                3.5,
                3.75,
                4.0,
            ],
            "v_lines": [1.0],
        }
        actual_dict = solvationplotinformation.create_plot_input_data_from_assocs(
            self.assoc_filename, "water", "x_label", "y_label"
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            with self.subTest(key=key):
                if key == "plot_axis_range":
                    np.testing.assert_array_almost_equal(
                        np.array(expected_dict[key]), np.array(actual_dict[key])
                    )
                elif "levels" == key or "_lines" in key:
                    np.testing.assert_array_equal(
                        np.array(expected_dict[key]), np.array(actual_dict[key])
                    )
                elif "_data" in key:
                    np.testing.assert_array_almost_equal(
                        expected_dict[key], actual_dict[key]
                    )
                else:
                    self.assertEqual(expected_dict[key], actual_dict[key])

    def test_create_plot_input_data(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {
            "x_data": np.array([[0, 0], [1, 1], [2, 2]]),
            "y_data": np.array([[0, 1], [0, 1], [0, 1]]),
            "z_data": np.array(
                [[-7.396882, -8.396882], [-8.396882, -9.021987], [-9.396882, -9.732173]]
            ),
            "x_label": "x_label",
            "y_label": "y_label",
            "plot_axis_range": (2.0, 0.0, 0.0, 1.0),
            "levels": [-11, -9, -7],
            "h_lines": [0.0, 0.25, 0.5, 0.75, 1.0],
            "v_lines": [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0],
        }
        actual_dict = solvationplotinformation.create_plot_input_data(
            self.epsilon_i_list,
            self.epsilon_j_list,
            298.0,
            1.0,
            np.array([0.5, 1.0]),
            "x_label",
            "y_label",
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            if key == "plot_axis_range":
                np.testing.assert_array_almost_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            elif "levels" == key or "_lines" in key:
                np.testing.assert_array_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            elif "_data" in key:
                np.testing.assert_array_almost_equal(
                    expected_dict[key], actual_dict[key]
                )
            else:
                self.assertEqual(expected_dict[key], actual_dict[key])

    def test_create_input_data_labels(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {
            "x_label": "x_label",
            "y_label": "y_label",
            "plot_axis_range": (2.0, 0.0, 0.0, 1.0),
            "levels": [-11, -9, -7],
            "h_lines": [0.0, 0.25, 0.5, 0.75, 1.0],
            "v_lines": [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0],
        }
        input_dict = {
            "x_data": np.array([[0, 0], [1, 1], [2, 2]]),
            "y_data": np.array([[0, 1], [0, 1], [0, 1]]),
            "z_data": np.array(
                [[-7.396882, -8.396882], [-8.396882, -9.021987], [-9.396882, -9.732173]]
            ),
        }
        actual_dict = solvationplotinformation.create_input_data_labels(
            input_dict, "x_label", "y_label"
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            if key == "plot_axis_range":
                np.testing.assert_array_almost_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            elif "levels" == key or "_lines" in key:
                np.testing.assert_array_equal(
                    np.array(expected_dict[key]), np.array(actual_dict[key])
                )
            else:
                self.assertEqual(expected_dict[key], actual_dict[key])

    def test_create_line_lists(self):
        """Test to se if expected values are returned.
        """
        expected_dict = {
            "h_lines": [0.0, 0.25, 0.5, 0.75, 1.0],
            "v_lines": [0.0, 0.25, 0.5, 0.75, 1.0, 1.25, 1.5, 1.75, 2.0],
        }
        actual_dict = solvationplotinformation.create_line_lists((2.0, 0.0, 0.0, 1.0))
        for key in expected_dict.keys():
            np.testing.assert_array_equal(
                np.array(expected_dict[key]), np.array(actual_dict[key])
            )
        expected_dict2 = {
            "h_lines": [x / 2.0 - 10.0 for x in range(31)],
            "v_lines": [x / 2.0 - 10.0 for x in range(31)],
        }
        actual_dict = solvationplotinformation.create_line_lists(
            (5.0, -10.0, -10.0, 5.0)
        )
        for key in expected_dict2.keys():
            np.testing.assert_array_equal(
                np.array(expected_dict2[key]), np.array(actual_dict[key])
            )

    def test_create_input_data_values(self):
        """Test to see if expected dictionary is produced.
        """
        expected_dict = {
            "x_data": np.array([[0, 0], [1, 1], [2, 2]]),
            "y_data": np.array([[0, 1], [0, 1], [0, 1]]),
            "z_data": np.array(
                [[-7.396882, -8.396882], [-8.396882, -9.021987], [-9.396882, -9.732173]]
            ),
        }
        actual_dict = solvationplotinformation.create_input_data_values(
            self.epsilon_i_list, self.epsilon_j_list, 298.0, 1.0, np.array([0.5, 1.0])
        )
        self.assertListEqual(sorted(expected_dict.keys()), sorted(actual_dict.keys()))
        for key in expected_dict.keys():
            np.testing.assert_array_almost_equal(expected_dict[key], actual_dict[key])

    def test_get_axis_range(self):
        """Test to see if expected axis range is returned.
        """
        expected_range = (2.0, 0.0, 0.0, 1.0)
        actual_range = solvationplotinformation.get_axis_range(
            self.epsilon_i_list, self.epsilon_j_list
        )
        np.testing.assert_array_almost_equal(
            np.array(expected_range), np.array(actual_range)
        )

    def test_create_meshgrids(self):
        """Test to see if expected arrays are produced.
        """
        expected_x = np.array([[0, 0], [1, 1], [2, 2]])
        expected_y = np.array([[0, 1], [0, 1], [0, 1]])
        actual_x, actual_y = solvationplotinformation.create_meshgrids(
            self.epsilon_i_list, self.epsilon_j_list
        )
        np.testing.assert_array_almost_equal(expected_x, actual_x)
        np.testing.assert_array_almost_equal(expected_y, actual_y)

    def test_calculate_association_energy_matrix(self):
        """Test to see if expected matrix is returned.
        """
        expected_array = np.array([[-9.021987]])
        actual_array = solvationplotinformation.calculate_association_energy_matrix(
            np.array([1.0]), np.array([1.0]), 298.0, 1.0, np.array([0.5, 1.0])
        )
        np.testing.assert_array_almost_equal(expected_array, actual_array)
